# Copyright (c) 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_utils import uuidutils
import six

from sahara import conductor as c
from sahara import context
from sahara.plugins import base as plugin_base
from sahara.service.edp.data_sources import manager as ds_manager
from sahara.service.edp.utils import shares as shares_service
from sahara.utils import remote


opts = [
    cfg.StrOpt('job_workflow_postfix',
               default='',
               help="Postfix for storing jobs in hdfs. Will be "
                    "added to '/user/<hdfs user>/' path.")
]

CONF = cfg.CONF
CONF.register_opts(opts)

conductor = c.API

# Prefix used to mark data_source name references in arg lists
DATA_SOURCE_PREFIX = "datasource://"

DATA_SOURCE_SUBST_NAME = "edp.substitute_data_source_for_name"
DATA_SOURCE_SUBST_UUID = "edp.substitute_data_source_for_uuid"


def get_plugin(cluster):
    return plugin_base.PLUGINS.get_plugin(cluster.plugin_name)


def get_data_source(ds_name):
    return ds_manager.DATA_SOURCES.get_data_source(ds_name)


def create_workflow_dir(where, path, job, use_uuid=None, chmod=""):

    if use_uuid is None:
        use_uuid = uuidutils.generate_uuid()

    constructed_dir = _append_slash_if_needed(path)
    constructed_dir += '%s/%s' % (job.name, use_uuid)
    with remote.get_remote(where) as r:
        if chmod:
            r.execute_command("mkdir -p -m %s %s" % (chmod, constructed_dir))
        else:
            r.execute_command("mkdir -p %s" % constructed_dir)
    return constructed_dir


def _get_data_source_urls(ds, cluster, job_exec_id):
    # returns a tuple (native_url, runtime_url)
    return get_data_source(ds.type).get_urls(ds.url, cluster, job_exec_id)


def get_input_output_data_sources(job_execution, job, data_source_urls,
                                  cluster=None):
    def _construct(ctx, ds_id):
        job_exec_id = job_execution.id
        source = conductor.data_source_get(ctx, ds_id)
        if source and source.id not in data_source_urls:
            data_source_urls[source.id] = _get_data_source_urls(source,
                                                                cluster,
                                                                job_exec_id)
        return source

    ctx = context.ctx()
    input_source = _construct(ctx, job_execution.input_id)
    output_source = _construct(ctx, job_execution.output_id)

    return input_source, output_source


def _append_slash_if_needed(path):
    if path[-1] != '/':
        path += '/'
    return path


def may_contain_data_source_refs(job_configs):

    def _check_data_source_ref_option(option):
        truth = job_configs and (
            job_configs.get('configs', {}).get(option))
        # Config values specified in the UI may be
        # passed as strings
        return truth in (True, 'True')

    return (
        _check_data_source_ref_option(DATA_SOURCE_SUBST_NAME),
        _check_data_source_ref_option(DATA_SOURCE_SUBST_UUID))


def _data_source_ref_search(job_configs, func, prune=lambda x: x):
    """Return a list of unique values in job_configs filtered by func().

    Loop over the 'args', 'configs' and 'params' elements in
    job_configs and return a list of all values for which
    func(value) is True.

    Optionally provide a 'prune' function that is applied
    to values before they are added to the return value.
    """
    args = set([prune(arg) for arg in job_configs.get(
        'args', []) if func(arg)])

    configs = set([prune(val) for val in six.itervalues(
        job_configs.get('configs', {})) if func(val)])

    params = set([prune(val) for val in six.itervalues(
        job_configs.get('params', {})) if func(val)])

    return list(args | configs | params)


def find_possible_data_source_refs_by_name(job_configs):
    """Find string values in job_configs starting with 'datasource://'.

    Loop over the 'args', 'configs', and 'params' elements of
    job_configs to find all values beginning with the prefix
    'datasource://'. Return a list of unique values with the prefix
    removed.

    Note that for 'configs' and 'params', which are dictionaries, only
    the values are considered and the keys are not relevant.
    """
    def startswith(arg):
        return isinstance(
            arg,
            six.string_types) and arg.startswith(DATA_SOURCE_PREFIX)
    return _data_source_ref_search(job_configs,
                                   startswith,
                                   prune=lambda x: x[len(DATA_SOURCE_PREFIX):])


def find_possible_data_source_refs_by_uuid(job_configs):
    """Find string values in job_configs which are uuids.

    Return a list of unique values in the 'args', 'configs', and 'params'
    elements of job_configs which have the form of a uuid.

    Note that for 'configs' and 'params', which are dictionaries, only
    the values are considered and the keys are not relevant.
    """
    return _data_source_ref_search(job_configs, uuidutils.is_uuid_like)


def resolve_data_source_references(job_configs,
                                   job_exec_id,
                                   data_source_urls,
                                   cluster=None):
    """Resolve possible data_source references in job_configs.

    Look for any string values in the 'args', 'configs', and 'params'
    elements of job_configs which start with 'datasource://' or have
    the form of a uuid.

    For values beginning with 'datasource://', strip off the prefix
    and search for a DataSource object with a name that matches the
    value.

    For values having the form of a uuid, search for a DataSource object
    with an id that matches the value.

    If a DataSource object is found for the value, replace the value
    with the URL from the DataSource object. If any DataSource objects
    are found which reference swift paths and contain credentials, set
    credential configuration values in job_configs (use the first set
    of swift credentials found).

    If no values are resolved, return an empty list and a reference
    to job_configs.

    If any values are resolved, return a list of the referenced
    data_source objects and a copy of job_configs with all of the
    references replaced with URLs.
    """
    by_name, by_uuid = may_contain_data_source_refs(job_configs)
    if not (by_name or by_uuid):
        return [], job_configs

    ctx = context.ctx()
    ds_seen = {}
    new_configs = {}

    def _resolve(value):
        kwargs = {}
        if by_name and isinstance(
                value,
                six.string_types) and value.startswith(DATA_SOURCE_PREFIX):
            value = value[len(DATA_SOURCE_PREFIX):]
            kwargs['name'] = value

        elif by_uuid and uuidutils.is_uuid_like(value):
            kwargs['id'] = value

        if kwargs:
            # Name and id are both unique constraints so if there
            # is more than 1 something is really wrong
            ds = conductor.data_source_get_all(ctx, **kwargs)
            if len(ds) == 1:
                ds = ds[0]
                ds_seen[ds.id] = ds
                if ds.id not in data_source_urls:
                    data_source_urls[ds.id] = _get_data_source_urls(
                        ds, cluster, job_exec_id)
                return data_source_urls[ds.id][1]
        return value

    # Loop over configs/params/args and look up each value as a data_source.
    # If we find it, replace the value. In all cases, we've produced a
    # copy which is not a FrozenClass type and can be updated.
    new_configs['configs'] = {
        k: _resolve(v) for k, v in six.iteritems(
            job_configs.get('configs', {}))}
    new_configs['params'] = {
        k: _resolve(v) for k, v in six.iteritems(
            job_configs.get('params', {}))}
    new_configs['args'] = [_resolve(a) for a in job_configs.get('args', [])]

    # If we didn't resolve anything we might as well return the original
    ds_seen = ds_seen.values()
    if not ds_seen:
        return [], job_configs

    # If there are proxy_configs we'll need to copy these, too,
    # so job_configs is complete
    if job_configs.get('proxy_configs'):
        new_configs['proxy_configs'] = {
            k: v for k, v in six.iteritems(job_configs.get('proxy_configs'))}

    return ds_seen, new_configs


def prepare_cluster_for_ds(data_sources, cluster, job_configs, ds_urls):
    for ds in data_sources:
        if ds:
            get_data_source(ds.type).prepare_cluster(
                ds, cluster, job_configs=job_configs,
                runtime_url=ds_urls[ds.id])


def to_url_dict(data_source_urls, runtime=False):
    idx = 1 if runtime else 0
    return {id: urls[idx] for id, urls in six.iteritems(data_source_urls)}


def mount_share_at_default_path(url, cluster):
    # Automount this share to the cluster with default path
    # url example: 'manila://ManilaShare-uuid/path_to_file'
    share_id = six.moves.urllib.parse.urlparse(url).netloc
    if cluster.shares:
        cluster_shares = [dict(s) for s in cluster.shares]
    else:
        cluster_shares = []

    needed_share = {
        'id': share_id,
        'path': shares_service.default_mount(share_id),
        'access_level': 'rw'
    }

    cluster_shares.append(needed_share)
    cluster = conductor.cluster_update(
        context.ctx(), cluster, {'shares': cluster_shares})
    shares_service.mount_shares(cluster)

    return shares_service.get_share_path(url, cluster.shares)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import mock

from oslo_utils import uuidutils

from sahara import conductor
from sahara import context
from sahara.tests.unit import base
from sahara.tests.unit.conductor import test_api
from sahara.utils import cluster_progress_ops as cpo


class FakeInstance(object):
    def __init__(self):
        self.id = uuidutils.generate_uuid()
        self.name = uuidutils.generate_uuid()
        self.cluster_id = uuidutils.generate_uuid()
        self.node_group_id = uuidutils.generate_uuid()
        self.instance_id = uuidutils.generate_uuid()
        self.instance_name = uuidutils.generate_uuid()


class ClusterProgressOpsTest(base.SaharaWithDbTestCase):
    def setUp(self):
        super(ClusterProgressOpsTest, self).setUp()
        self.api = conductor.API

    def _make_sample(self):
        ctx = context.ctx()
        cluster = self.api.cluster_create(ctx, test_api.SAMPLE_CLUSTER)
        return ctx, cluster

    def test_update_provisioning_steps(self):
        ctx, cluster = self._make_sample()

        step_id1 = self.api.cluster_provision_step_add(ctx, cluster.id, {
            "step_name": "some_name1",
            "total": 2,
        })

        self.api.cluster_event_add(ctx, step_id1, {
            "event_info": "INFO",
            "successful": True
        })

        self.api.cluster_provision_progress_update(ctx, cluster.id)

        # check that we have correct provision step

        result_cluster = self.api.cluster_get(ctx, cluster.id)
        result_step = result_cluster.provision_progress[0]

        self.assertIsNone(result_step.successful)

        # check updating in case of successful provision step

        self.api.cluster_event_add(ctx, step_id1, {
            "event_info": "INFO",
            "successful": True
        })

        self.api.cluster_provision_progress_update(ctx, cluster.id)

        result_cluster = self.api.cluster_get(ctx, cluster.id)
        result_step = result_cluster.provision_progress[0]

        self.assertTrue(result_step.successful)

        # check updating in case of failed provision step

        step_id2 = self.api.cluster_provision_step_add(ctx, cluster.id, {
            "step_name": "some_name1",
            "total": 2,
        })

        self.api.cluster_event_add(ctx, step_id2, {
            "event_info": "INFO",
            "successful": False,
        })

        self.api.cluster_provision_progress_update(ctx, cluster.id)

        result_cluster = self.api.cluster_get(ctx, cluster.id)

        for step in result_cluster.provision_progress:
            if step.id == step_id2:
                self.assertFalse(step.successful)

        # check that it's possible to add provision step after failed step
        step_id3 = cpo.add_provisioning_step(cluster.id, "some_name", 2)

        self.assertEqual(
            step_id3, cpo.get_current_provisioning_step(cluster.id))

    def test_get_cluster_events(self):
        ctx, cluster = self._make_sample()

        step_id1 = self.api.cluster_provision_step_add(ctx, cluster.id, {
            'step_name': "some_name1",
            'total': 3,
        })
        step_id2 = self.api.cluster_provision_step_add(ctx, cluster.id, {
            'step_name': "some_name",
            'total': 2,
        })

        self.api.cluster_event_add(ctx, step_id1, {
            "event_info": "INFO",
            'successful': True,
        })

        self.api.cluster_event_add(ctx, step_id2, {
            "event_info": "INFO",
            'successful': True,
        })
        cluster = self.api.cluster_get(context.ctx(), cluster.id, True)
        for step in cluster.provision_progress:
            self.assertEqual(1, len(step.events))

    def _make_checks(self, instance_info, sleep=True):
        ctx = context.ctx()

        if sleep:
            context.sleep(2)

        current_instance_info = ctx.current_instance_info
        self.assertEqual(instance_info, current_instance_info)

    def test_instance_context_manager(self):
        fake_instances = [FakeInstance() for _ in range(50)]

        # check that InstanceContextManager works fine sequentially

        for instance in fake_instances:
            info = context.InstanceInfo(
                None, instance.id, instance.name, None)
            with context.InstanceInfoManager(info):
                self._make_checks(info, sleep=False)

        # check that InstanceContextManager works fine in parallel

        with context.ThreadGroup() as tg:
            for instance in fake_instances:
                info = context.InstanceInfo(
                    None, instance.id, instance.name, None)
                with context.InstanceInfoManager(info):
                    tg.spawn("make_checks", self._make_checks, info)

    @cpo.event_wrapper(True)
    def _do_nothing(self):
        pass

    @mock.patch('sahara.utils.cluster_progress_ops._find_in_args')
    @mock.patch('sahara.utils.cluster.check_cluster_exists')
    def test_event_wrapper(self, p_check_cluster_exists, p_find):
        self.override_config("disable_event_log", True)
        self._do_nothing()

        self.assertEqual(0, p_find.call_count)

        self.override_config("disable_event_log", False)
        p_find.return_value = FakeInstance()
        p_check_cluster_exists.return_value = False
        self._do_nothing()

        self.assertEqual(1, p_find.call_count)
        self.assertEqual(1, p_check_cluster_exists.call_count)

    def test_cluster_get_with_events(self):
        ctx, cluster = self._make_sample()

        step_id = cpo.add_provisioning_step(cluster.id, "Some name", 3)
        self.api.cluster_event_add(ctx, step_id, {
            'event_info': "INFO", 'successful': True})
        cluster = self.api.cluster_get(ctx, cluster.id, True)

        steps = cluster.provision_progress
        step = steps[0]
        self.assertEqual("Some name", step.step_name)
        self.assertEqual(3, step.total)
        self.assertEqual("INFO", step.events[0].event_info)

    @mock.patch('sahara.context.ctx')
    @mock.patch(
        'sahara.utils.cluster_progress_ops.get_current_provisioning_step',
        return_value='step_id')
    @mock.patch('sahara.utils.cluster_progress_ops.conductor')
    def test_add_successful_event(self, conductor, get_step, ctx):
        instance = FakeInstance()

        self.override_config("disable_event_log", True)
        cpo.add_successful_event(instance)
        self.assertEqual(0, conductor.cluster_event_add.call_count)

        self.override_config("disable_event_log", False)
        cpo.add_successful_event(instance)
        self.assertEqual(1, conductor.cluster_event_add.call_count)
        args, kwargs = conductor.cluster_event_add.call_args
        self.assertEqual('step_id', args[1])
        req_dict = {
            'successful': True,
            'node_group_id': instance.node_group_id,
            'instance_id': instance.instance_id,
            'instance_name': instance.instance_name,
            'event_info': None,
        }
        self.assertEqual(req_dict, args[2])

    @mock.patch('sahara.context.ctx')
    @mock.patch(
        'sahara.utils.cluster_progress_ops.get_current_provisioning_step',
        return_value='step_id')
    @mock.patch('sahara.utils.cluster_progress_ops.conductor')
    def test_add_fail_event(self, conductor, get_step, ctx):
        instance = FakeInstance()

        self.override_config("disable_event_log", True)
        cpo.add_fail_event(instance, Exception('oops'))
        self.assertEqual(0, conductor.cluster_event_add.call_count)

        self.override_config("disable_event_log", False)
        cpo.add_fail_event(instance, Exception('oops'))
        self.assertEqual(1, conductor.cluster_event_add.call_count)
        args, kwargs = conductor.cluster_event_add.call_args
        self.assertEqual('step_id', args[1])
        req_dict = {
            'successful': False,
            'node_group_id': instance.node_group_id,
            'instance_id': instance.instance_id,
            'instance_name': instance.instance_name,
            'event_info': 'oops',
        }
        self.assertEqual(req_dict, args[2])



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
from oslo_middleware import base
from oslo_utils import strutils
from oslo_utils import uuidutils
import webob
import webob.exc as ex

from sahara.i18n import _


LOG = logging.getLogger(__name__)


class AuthValidator(base.Middleware):

    """Handles token auth results and tenants."""

    @webob.dec.wsgify
    def __call__(self, req):
        """Ensures that tenants in url and token are equal.

        Handle incoming request by checking tenant info prom the headers and
        url ({tenant_id} url attribute).

        Pass request downstream on success.
        Reject request if tenant_id from headers not equals to tenant_id from
        url.
        """
        token_tenant = req.environ.get("HTTP_X_TENANT_ID")
        if not token_tenant:
            LOG.warning("Can't get tenant_id from env")
            raise ex.HTTPServiceUnavailable()

        path = req.environ['PATH_INFO']
        if path != '/':
            try:
                version, possibly_url_tenant, rest = (
                    strutils.split_path(path, 2, 3, True)
                )
            except ValueError:
                LOG.warning("Incorrect path: {path}".format(path=path))
                raise ex.HTTPNotFound(_("Incorrect path"))

            if uuidutils.is_uuid_like(possibly_url_tenant):
                url_tenant = possibly_url_tenant
                if token_tenant != url_tenant:
                    LOG.debug("Unauthorized: token tenant != requested tenant")
                    raise ex.HTTPUnauthorized(
                        _('Token tenant != requested tenant'))
        return self.application



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import mock

from cinderclient import exceptions as cinder_exc
from heatclient import exc as heat_exc
from keystoneauth1 import exceptions as keystone_exc
from neutronclient.common import exceptions as neutron_exc
from novaclient import exceptions as nova_exc

from sahara import exceptions as sahara_exc
from sahara.tests.unit import base as testbase
from sahara.utils.openstack import base


class TestBase(testbase.SaharaTestCase):

    def test_url_for_regions(self):
        service_catalog = (
            '[{"endpoints": '
            '  [{"adminURL": "http://192.168.0.5:8774/v2", '
            '    "region": "RegionOne", '
            '    "id": "83d12c9ad2d647ecab7cbe91adb8666b", '
            '    "internalURL": "http://192.168.0.5:8774/v2", '
            '    "publicURL": "http://172.18.184.5:8774/v2"}, '
            '   {"adminURL": "http://192.168.0.6:8774/v2", '
            '    "region": "RegionTwo", '
            '    "id": "07c5a555176246c783d8f0497c98537b", '
            '    "internalURL": "http://192.168.0.6:8774/v2", '
            '    "publicURL": "http://172.18.184.6:8774/v2"}], '
            '  "endpoints_links": [], '
            '  "type": "compute", '
            '  "name": "nova"}]')

        self.override_config("os_region_name", "RegionOne")
        self.assertEqual("http://192.168.0.5:8774/v2",
                         base.url_for(service_catalog, "compute"))

        self.override_config("os_region_name", "RegionTwo")
        self.assertEqual("http://192.168.0.6:8774/v2",
                         base.url_for(service_catalog, "compute"))


class AuthUrlTest(testbase.SaharaTestCase):

    def test_retrieve_auth_url_api_v3(self):
        self.override_config('use_identity_api_v3', True)
        correct = "https://127.0.0.1:8080/v3"

        def _assert(uri):
            self.override_config('auth_url', uri, 'trustee')
            self.assertEqual(correct, base.retrieve_auth_url())

        _assert("%s/" % correct)
        _assert("https://127.0.0.1:8080")
        _assert("https://127.0.0.1:8080/")
        _assert("https://127.0.0.1:8080/v2.0")
        _assert("https://127.0.0.1:8080/v2.0/")
        _assert("https://127.0.0.1:8080/v3")
        _assert("https://127.0.0.1:8080/v3/")

    @mock.patch("sahara.utils.openstack.base.url_for")
    def test_retrieve_auth_url_api_v3_without_port(self, mock_url_for):
        self.override_config('use_identity_api_v3', True)
        self.setup_context(service_catalog=True)
        correct = "https://127.0.0.1/v3"

        def _assert(uri):
            mock_url_for.return_value = uri
            self.assertEqual(correct, base.retrieve_auth_url())

        _assert("%s/" % correct)
        _assert("https://127.0.0.1")
        _assert("https://127.0.0.1/")
        _assert("https://127.0.0.1/v2.0")
        _assert("https://127.0.0.1/v2.0/")
        _assert("https://127.0.0.1/v3")
        _assert("https://127.0.0.1/v3/")

    @mock.patch("sahara.utils.openstack.base.url_for")
    def test_retrieve_auth_url_api_v3_path_present(self, mock_url_for):
        self.override_config('use_identity_api_v3', True)
        self.setup_context(service_catalog=True)
        correct = "https://127.0.0.1/identity/v3"

        def _assert(uri):
            mock_url_for.return_value = uri
            self.assertEqual(correct, base.retrieve_auth_url())

        _assert("%s" % correct)
        _assert("%s/" % correct)
        _assert("https://127.0.0.1/identity")
        _assert("https://127.0.0.1/identity/")

    def test_retrieve_auth_url_api_v20(self):
        self.override_config('use_identity_api_v3', False)
        correct = "https://127.0.0.1:8080/v2.0"

        def _assert(uri):
            self.override_config('auth_url', uri, 'trustee')
            self.assertEqual(correct, base.retrieve_auth_url())

        _assert("%s/" % correct)
        _assert("https://127.0.0.1:8080")
        _assert("https://127.0.0.1:8080/")
        _assert("https://127.0.0.1:8080/v2.0")
        _assert("https://127.0.0.1:8080/v2.0/")
        _assert("https://127.0.0.1:8080/v3")
        _assert("https://127.0.0.1:8080/v3/")

    @mock.patch("sahara.utils.openstack.base.url_for")
    def test_retrieve_auth_url_api_v20_without_port(self, mock_url_for):
        self.override_config('use_identity_api_v3', False)
        self.setup_context(service_catalog=True)
        correct = "https://127.0.0.1/v2.0"

        def _assert(uri):
            mock_url_for.return_value = uri
            self.assertEqual(correct, base.retrieve_auth_url())

        _assert("%s/" % correct)
        _assert("https://127.0.0.1")
        _assert("https://127.0.0.1/")
        _assert("https://127.0.0.1/v2.0")
        _assert("https://127.0.0.1/v2.0/")
        _assert("https://127.0.0.1/v3")
        _assert("https://127.0.0.1/v3/")


class ExecuteWithRetryTest(testbase.SaharaTestCase):

    def setUp(self):
        super(ExecuteWithRetryTest, self).setUp()
        self.fake_client_call = mock.MagicMock()
        self.fake_client_call.__name__ = 'fake_client_call'
        self.override_config('retries_number', 2, 'retries')

    @mock.patch('sahara.context.sleep')
    def _check_error_without_retry(self, error, code, m_sleep):
        self.fake_client_call.side_effect = error(code)

        self.assertRaises(error, base.execute_with_retries,
                          self.fake_client_call)
        self.assertEqual(1, self.fake_client_call.call_count)
        self.fake_client_call.reset_mock()

    @mock.patch('sahara.context.sleep')
    def _check_error_with_retry(self, error, code, m_sleep):
        self.fake_client_call.side_effect = error(code)

        self.assertRaises(sahara_exc.MaxRetriesExceeded,
                          base.execute_with_retries, self.fake_client_call)
        self.assertEqual(3, self.fake_client_call.call_count)
        self.fake_client_call.reset_mock()

    def test_novaclient_calls_without_retry(self):
        # check that following errors will not be retried
        self._check_error_without_retry(nova_exc.BadRequest, 400)
        self._check_error_without_retry(nova_exc.Unauthorized, 401)
        self._check_error_without_retry(nova_exc.Forbidden, 403)
        self._check_error_without_retry(nova_exc.NotFound, 404)
        self._check_error_without_retry(nova_exc.MethodNotAllowed, 405)
        self._check_error_without_retry(nova_exc.Conflict, 409)
        self._check_error_without_retry(nova_exc.HTTPNotImplemented, 501)

    def test_novaclient_calls_with_retry(self):
        # check that following errors will be retried
        self._check_error_with_retry(nova_exc.OverLimit, 413)
        self._check_error_with_retry(nova_exc.RateLimit, 429)

    def test_cinderclient_calls_without_retry(self):
        # check that following errors will not be retried
        self._check_error_without_retry(cinder_exc.BadRequest, 400)
        self._check_error_without_retry(cinder_exc.Unauthorized, 401)
        self._check_error_without_retry(cinder_exc.Forbidden, 403)
        self._check_error_without_retry(cinder_exc.NotFound, 404)
        self._check_error_without_retry(nova_exc.HTTPNotImplemented, 501)

    def test_cinderclient_calls_with_retry(self):
        # check that following error will be retried
        self._check_error_with_retry(cinder_exc.OverLimit, 413)

    def test_neutronclient_calls_without_retry(self):
        # check that following errors will not be retried
        # neutron exception expects string in constructor
        self._check_error_without_retry(neutron_exc.BadRequest, "400")
        self._check_error_without_retry(neutron_exc.Forbidden, "403")
        self._check_error_without_retry(neutron_exc.NotFound, "404")
        self._check_error_without_retry(neutron_exc.Conflict, "409")

    def test_neutronclient_calls_with_retry(self):
        # check that following errors will be retried
        # neutron exception expects string in constructor
        self._check_error_with_retry(neutron_exc.InternalServerError, "500")
        self._check_error_with_retry(neutron_exc.ServiceUnavailable, "503")

    def test_heatclient_calls_without_retry(self):
        # check that following errors will not be retried
        self._check_error_without_retry(heat_exc.HTTPBadRequest, 400)
        self._check_error_without_retry(heat_exc.HTTPUnauthorized, 401)
        self._check_error_without_retry(heat_exc.HTTPForbidden, 403)
        self._check_error_without_retry(heat_exc.HTTPNotFound, 404)
        self._check_error_without_retry(heat_exc.HTTPMethodNotAllowed, 405)
        self._check_error_without_retry(heat_exc.HTTPConflict, 409)
        self._check_error_without_retry(heat_exc.HTTPUnsupported, 415)
        self._check_error_without_retry(heat_exc.HTTPNotImplemented, 501)

    def test_heatclient_calls_with_retry(self):
        # check that following errors will be retried
        self._check_error_with_retry(heat_exc.HTTPInternalServerError, 500)
        self._check_error_with_retry(heat_exc.HTTPBadGateway, 502)
        self._check_error_with_retry(heat_exc.HTTPServiceUnavailable, 503)

    def test_keystoneclient_calls_without_retry(self):
        # check that following errors will not be retried
        self._check_error_without_retry(keystone_exc.BadRequest, 400)
        self._check_error_without_retry(keystone_exc.Unauthorized, 401)
        self._check_error_without_retry(keystone_exc.Forbidden, 403)
        self._check_error_without_retry(keystone_exc.NotFound, 404)
        self._check_error_without_retry(keystone_exc.MethodNotAllowed, 405)
        self._check_error_without_retry(keystone_exc.Conflict, 409)
        self._check_error_without_retry(keystone_exc.UnsupportedMediaType, 415)
        self._check_error_without_retry(keystone_exc.HttpNotImplemented, 501)

    def test_keystoneclient_calls_with_retry(self):
        # check that following errors will be retried
        self._check_error_with_retry(keystone_exc.RequestTimeout, 408)
        self._check_error_with_retry(keystone_exc.InternalServerError, 500)
        self._check_error_with_retry(keystone_exc.BadGateway, 502)
        self._check_error_with_retry(keystone_exc.ServiceUnavailable, 503)
        self._check_error_with_retry(keystone_exc.GatewayTimeout, 504)



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_config import cfg
from oslo_log import log as logging
import six
import six.moves.urllib.parse as urlparse
from stevedore import enabled

from sahara import conductor as cond
from sahara import exceptions as ex
from sahara.i18n import _

conductor = cond.API

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class JobBinaryManager(object):
    def __init__(self):
        self.job_binaries = {}
        self._load_job_binaries()

    def _load_job_binaries(self):
        config_jb = CONF.job_binary_types
        extension_manager = enabled.EnabledExtensionManager(
            check_func=lambda ext: ext.name in config_jb,
            namespace='sahara.job_binary.types',
            invoke_on_load=True
        )

        for ext in extension_manager.extensions:
            if ext.name in self.job_binaries:
                raise ex.ConfigurationError(
                    _("Job binary with name '%s' already exists.") %
                    ext.name)
            ext.obj.name = ext.name
            self.job_binaries[ext.name] = ext.obj
            LOG.info("Job binary name {jb_name} loaded {entry_point}".format(
                jb_name=ext.name, entry_point=ext.entry_point_target))

        if len(self.job_binaries) < len(config_jb):
            loaded_jb = set(six.iterkeys(self.job_binaries))
            requested_jb = set(config_jb)
            raise ex.ConfigurationError(
                _("Job binaries couldn't be loaded: %s") %
                ", ".join(requested_jb - loaded_jb))

    def get_job_binaries(self):
        config_jb = CONF.job_binary_types
        return [self.get_job_binary(name).name for name in config_jb]

    def get_job_binary(self, name):
        res = self.job_binaries.get(name)
        if res is None:
            raise ex.InvalidDataException(_("Invalid job binary"))
        return res

    def get_job_binary_by_url(self, url):
        url = urlparse.urlparse(url)
        if not url.scheme:
            raise ex.InvalidDataException(
                _("Job binary url must have a scheme"))
        return self.get_job_binary(url.scheme)


JOB_BINARIES = None


def setup_job_binaries():
    global JOB_BINARIES
    JOB_BINARIES = JobBinaryManager()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# File contains plugins opts to avoid cyclic imports issue

from oslo_config import cfg

opts = [
    cfg.ListOpt('plugins',
                default=['vanilla', 'spark', 'cdh', 'ambari', 'storm', 'mapr'],
                help='List of plugins to be loaded. Sahara preserves the '
                     'order of the list when returning it.'),
]

CONF = cfg.CONF
CONF.register_opts(opts)



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_utils import uuidutils
import six.moves.urllib.parse as urlparse

from sahara import conductor as c
from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.edp.job_binaries.base import JobBinaryType
import sahara.service.validations.edp.base as b

CONF = cfg.CONF
conductor = c.API


class InternalDBType(JobBinaryType):
    def copy_binary_to_cluster(self, job_binary, **kwargs):
        # url example: 'internal-db://JobBinaryInternal-UUID'
        r = kwargs.pop('remote')

        dst = self._generate_valid_path(job_binary)
        raw = self.get_raw_data(job_binary, **kwargs)

        r.write_file_to(dst, raw)
        return dst

    def get_raw_data(self, job_binary, **kwargs):
        context = kwargs.pop('context')
        # url example: 'internal-db://JobBinaryInternal-UUID'
        binary_internal_id = job_binary.url[len("internal-db://"):]
        return conductor.job_binary_internal_get_raw_data(context,
                                                          binary_internal_id)

    def validate_job_location_format(self, url):
        try:
            self._validate_url(url)
        except ex.InvalidDataException:
            return False
        return True

    def validate(self, data, **kwargs):
        self._validate_url(data['url'])
        internal_uid = data['url'].replace("internal-db://", '')
        b.check_job_binary_internal_exists(internal_uid)

    def _validate_url(self, url):
        if len(url) == 0:
            raise ex.InvalidDataException(
                _("Internal data base url must not be empty"))
        url = urlparse.urlparse(url)
        if url.scheme != "internal-db":
            raise ex.InvalidDataException(
                _("URL scheme must be 'internal-db'"))
        if not uuidutils.is_uuid_like(url.netloc):
            raise ex.InvalidDataException(
                _("Internal data base url netloc must be a uuid"))



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import shlex
from unittest import mock

import testtools

from sahara import exceptions as ex
from sahara.tests.unit import base
from sahara.utils import ssh_remote


class TestEscapeQuotes(testtools.TestCase):
    def test_escape_quotes(self):
        s = ssh_remote._escape_quotes('echo "\\"Hello, world!\\""')
        self.assertEqual(r'echo \"\\\"Hello, world!\\\"\"', s)


class TestGetOsDistrib(testtools.TestCase):
    @mock.patch('sahara.utils.ssh_remote._execute_command',
                return_value=[1, 'Ubuntu'])
    @mock.patch('sahara.utils.ssh_remote._get_python_to_execute',
                return_value='python3')
    def test_get_os_distrib(self, python, p_execute_command):
        d = ssh_remote._get_os_distrib()
        p_execute_command.assert_called_once_with(
            ('printf "import platform\nprint(platform.linux_distribution('
             'full_distribution_name=0)[0])" | python3'),
            run_as_root=False)
        self.assertEqual('ubuntu', d)


class TestInstallPackages(testtools.TestCase):
    @mock.patch('sahara.utils.ssh_remote._get_os_version')
    @mock.patch('sahara.utils.ssh_remote._get_os_distrib')
    @mock.patch('sahara.utils.ssh_remote._execute_command')
    def test_install_packages(self, p_execute_command, p_get_os_distrib,
                              p_get_os_version):
        packages = ('git', 'emacs', 'tree')

        # test ubuntu
        p_get_os_distrib.return_value = 'ubuntu'
        ssh_remote._install_packages(packages)
        p_execute_command.assert_called_with(
            'RUNLEVEL=1 apt-get install -y git emacs tree', run_as_root=True)

        # test centos
        p_get_os_distrib.return_value = 'centos'
        ssh_remote._install_packages(packages)
        p_execute_command.assert_called_with(
            'yum install -y git emacs tree',
            run_as_root=True)

        # test fedora < 22
        p_get_os_distrib.return_value = 'fedora'
        p_get_os_version.return_value = 20
        ssh_remote._install_packages(packages)
        p_execute_command.assert_called_with(
            'yum install -y git emacs tree',
            run_as_root=True)

        # test fedora >=22
        p_get_os_distrib.return_value = 'fedora'
        p_get_os_version.return_value = 23
        ssh_remote._install_packages(packages)
        p_execute_command.assert_called_with(
            'dnf install -y git emacs tree',
            run_as_root=True)

        # test redhat
        p_get_os_distrib.return_value = 'redhat'
        ssh_remote._install_packages(packages)
        p_execute_command.assert_called_with(
            'yum install -y git emacs tree',
            run_as_root=True)

    @mock.patch('sahara.utils.ssh_remote._get_os_distrib',
                return_value='windows me')
    def test_install_packages_bad(self, p_get_os_distrib):
        with testtools.ExpectedException(
                ex.NotImplementedException,
                'Package Installation is not implemented for OS windows me.*'):
            ssh_remote._install_packages(('git', 'emacs', 'tree'))


class TestUpdateRepository(testtools.TestCase):
    @mock.patch('sahara.utils.ssh_remote._get_os_version')
    @mock.patch('sahara.utils.ssh_remote._get_os_distrib')
    @mock.patch('sahara.utils.ssh_remote._execute_command')
    def test_update_repository(self, p_execute_command, p_get_os_distrib,
                               p_get_os_version):
        # test ubuntu
        p_get_os_distrib.return_value = 'ubuntu'
        ssh_remote._update_repository()
        p_execute_command.assert_called_with(
            'apt-get update', run_as_root=True)

        # test centos
        p_get_os_distrib.return_value = 'centos'
        ssh_remote._update_repository()
        p_execute_command.assert_called_with(
            'yum clean all', run_as_root=True)

        # test fedora < 22
        p_get_os_distrib.return_value = 'fedora'
        p_get_os_version.return_value = 20
        ssh_remote._update_repository()
        p_execute_command.assert_called_with(
            'yum clean all', run_as_root=True)

        # test fedora >=22
        p_get_os_distrib.return_value = 'fedora'
        p_get_os_version.return_value = 23
        ssh_remote._update_repository()
        p_execute_command.assert_called_with(
            'dnf clean all', run_as_root=True)
        # test redhat
        p_get_os_distrib.return_value = 'redhat'
        ssh_remote._update_repository()
        p_execute_command.assert_called_with(
            'yum clean all', run_as_root=True)

    @mock.patch('sahara.utils.ssh_remote._get_os_distrib',
                return_value='windows me')
    def test_update_repository_bad(self, p_get_os_distrib):
        with testtools.ExpectedException(
                ex.NotImplementedException,
                'Repository Update is not implemented for OS windows me.*'):
            ssh_remote._update_repository()


class FakeCluster(object):
    def __init__(self, priv_key):
        self.management_private_key = priv_key
        self.neutron_management_network = 'network1'

    def has_proxy_gateway(self):
        return False

    def get_proxy_gateway_node(self):
        return None


class FakeNodeGroup(object):
    def __init__(self, user, priv_key):
        self.image_username = user
        self.cluster = FakeCluster(priv_key)
        self.floating_ip_pool = 'public'


class FakeInstance(object):
    def __init__(self, inst_name, inst_id, management_ip, internal_ip, user,
                 priv_key):
        self.instance_name = inst_name
        self.instance_id = inst_id
        self.management_ip = management_ip
        self.internal_ip = internal_ip
        self.node_group = FakeNodeGroup(user, priv_key)

    @property
    def cluster(self):
        return self.node_group.cluster


class TestInstanceInteropHelper(base.SaharaTestCase):
    def setUp(self):
        super(TestInstanceInteropHelper, self).setUp()

        p_sma = mock.patch('sahara.utils.ssh_remote._acquire_remote_semaphore')
        p_sma.start()
        p_smr = mock.patch('sahara.utils.ssh_remote._release_remote_semaphore')
        p_smr.start()

        p_neutron_router = mock.patch(
            'sahara.utils.openstack.neutron.NeutronClient.get_router',
            return_value='fakerouter')
        p_neutron_router.start()
        # During tests subprocesses are not used (because _sahara-subprocess
        # is not installed in /bin and Mock objects cannot be pickled).
        p_start_subp = mock.patch('sahara.utils.procutils.start_subprocess',
                                  return_value=42)
        p_start_subp.start()
        p_run_subp = mock.patch('sahara.utils.procutils.run_in_subprocess')
        self.run_in_subprocess = p_run_subp.start()
        p_shut_subp = mock.patch('sahara.utils.procutils.shutdown_subprocess')
        p_shut_subp.start()

        self.patchers = [p_sma, p_smr, p_neutron_router, p_start_subp,
                         p_run_subp, p_shut_subp]

    def tearDown(self):
        for patcher in self.patchers:
            patcher.stop()
        super(TestInstanceInteropHelper, self).tearDown()

    def setup_context(self, username="test_user", tenant_id="tenant_1",
                      token="test_auth_token", tenant_name='test_tenant',
                      **kwargs):
        service_catalog = '''[
            { "type": "network",
              "endpoints": [ { "region": "RegionOne",
                               "publicURL": "http://localhost/" } ] } ]'''
        super(TestInstanceInteropHelper, self).setup_context(
            username=username, tenant_id=tenant_id, token=token,
            tenant_name=tenant_name, service_catalog=service_catalog, **kwargs)

    # When use_floating_ips=True, no proxy should be used: _connect is called
    # with proxy=None and ProxiedHTTPAdapter is not used.
    @mock.patch('sahara.utils.ssh_remote.ProxiedHTTPAdapter')
    def test_use_floating_ips(self, p_adapter):
        self.override_config('use_floating_ips', True)

        instance = FakeInstance('inst1', '123', '10.0.0.1', '10.0.0.1',
                                'user1', 'key1')
        remote = ssh_remote.InstanceInteropHelper(instance)

        # Test SSH
        remote.execute_command('/bin/true')
        self.run_in_subprocess.assert_any_call(
            42, ssh_remote._connect, ('10.0.0.1', 'user1', 'key1',
                                      None, None, None))
        # Test HTTP
        remote.get_http_client(8080)
        self.assertFalse(p_adapter.called)

    # When use_floating_ips=False and use_namespaces=True, a netcat socket
    # created with 'ip netns exec qrouter-...' should be used to access
    # instances.
    @mock.patch("sahara.service.trusts.get_os_admin_auth_plugin")
    @mock.patch("sahara.utils.openstack.keystone.token_auth")
    @mock.patch('sahara.utils.ssh_remote._simple_exec_func')
    @mock.patch('sahara.utils.ssh_remote.ProxiedHTTPAdapter')
    def test_use_namespaces(self, p_adapter, p_simple_exec_func, token_auth,
                            use_os_admin):
        self.override_config('use_floating_ips', False)
        self.override_config('use_namespaces', True)

        instance = FakeInstance('inst2', '123', '10.0.0.2', '10.0.0.2',
                                'user2', 'key2')
        remote = ssh_remote.InstanceInteropHelper(instance)

        # Test SSH
        remote.execute_command('/bin/true')
        self.run_in_subprocess.assert_any_call(
            42, ssh_remote._connect,
            ('10.0.0.2', 'user2', 'key2',
             'ip netns exec qrouter-fakerouter nc 10.0.0.2 22', None, None))
        # Test HTTP
        remote.get_http_client(8080)
        p_adapter.assert_called_once_with(
            p_simple_exec_func(),
            '10.0.0.2', 8080)
        p_simple_exec_func.assert_any_call(
            shlex.split('ip netns exec qrouter-fakerouter nc 10.0.0.2 8080'))

    # When proxy_command is set, a user-defined netcat socket should be used to
    # access instances.
    @mock.patch('sahara.utils.ssh_remote._simple_exec_func')
    @mock.patch('sahara.utils.ssh_remote.ProxiedHTTPAdapter')
    def test_proxy_command(self, p_adapter, p_simple_exec_func):
        self.override_config('proxy_command', 'ssh fakerelay nc {host} {port}')

        instance = FakeInstance('inst3', '123', '10.0.0.3', '10.0.0.3',
                                'user3', 'key3')
        remote = ssh_remote.InstanceInteropHelper(instance)

        # Test SSH
        remote.execute_command('/bin/true')
        self.run_in_subprocess.assert_any_call(
            42, ssh_remote._connect,
            ('10.0.0.3', 'user3', 'key3', 'ssh fakerelay nc 10.0.0.3 22',
             None, None))
        # Test HTTP
        remote.get_http_client(8080)
        p_adapter.assert_called_once_with(
            p_simple_exec_func(), '10.0.0.3', 8080)
        p_simple_exec_func.assert_any_call(
            shlex.split('ssh fakerelay nc 10.0.0.3 8080'))

    @mock.patch('sahara.utils.ssh_remote._simple_exec_func')
    @mock.patch('sahara.utils.ssh_remote.ProxiedHTTPAdapter')
    def test_proxy_command_internal_ip(self, p_adapter, p_simple_exec_func):
        self.override_config('proxy_command', 'ssh fakerelay nc {host} {port}')
        self.override_config('proxy_command_use_internal_ip', True)

        instance = FakeInstance('inst3', '123', '10.0.0.3', '10.0.0.4',
                                'user3', 'key3')
        remote = ssh_remote.InstanceInteropHelper(instance)

        # Test SSH
        remote.execute_command('/bin/true')
        self.run_in_subprocess.assert_any_call(
            42, ssh_remote._connect,
            ('10.0.0.4', 'user3', 'key3', 'ssh fakerelay nc 10.0.0.4 22',
             None, None))
        # Test HTTP
        remote.get_http_client(8080)
        p_adapter.assert_called_once_with(
            p_simple_exec_func(), '10.0.0.4', 8080)
        p_simple_exec_func.assert_any_call(
            shlex.split('ssh fakerelay nc 10.0.0.4 8080'))

    def test_proxy_command_bad(self):
        self.override_config('proxy_command', '{bad_kw} nc {host} {port}')

        instance = FakeInstance('inst4', '123', '10.0.0.4', '10.0.0.4',
                                'user4', 'key4')
        remote = ssh_remote.InstanceInteropHelper(instance)

        # Test SSH
        self.assertRaises(ex.SystemError, remote.execute_command, '/bin/true')
        # Test HTTP
        self.assertRaises(ex.SystemError, remote.get_http_client, 8080)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    def test_get_os_distrib(self, p_run_s):
        instance = FakeInstance('inst4', '123', '10.0.0.4', '10.0.0.4',
                                'user4', 'key4')
        remote = ssh_remote.InstanceInteropHelper(instance)

        remote.get_os_distrib()
        p_run_s.assert_called_with(ssh_remote._get_os_distrib,
                                   None, "get_os_distrib")

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_install_packages(self, p_log_command, p_run_s):
        instance = FakeInstance('inst5', '123', '10.0.0.5', '10.0.0.5',
                                'user5', 'key5')
        remote = ssh_remote.InstanceInteropHelper(instance)

        packages = ['pkg1', 'pkg2']
        remote.install_packages(packages)
        description = 'Installing packages "%s"' % list(packages)
        p_run_s.assert_called_once_with(
            ssh_remote._install_packages, None, description, packages)

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_update_repository(self, p_log_command, p_run_s):
        instance = FakeInstance('inst6', '123', '10.0.0.6', '10.0.0.6',
                                'user6', 'key6')
        remote = ssh_remote.InstanceInteropHelper(instance)

        remote.update_repository()
        p_run_s.assert_called_once_with(ssh_remote._update_repository,
                                        None, 'Updating repository')

        p_log_command.assert_called_with('Updating repository')

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_write_file_to(self, p_log_command, p_run_s):
        instance = FakeInstance('inst7', '123', '10.0.0.7', '10.0.0.7',
                                'user7', 'key7')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'Writing file "file"'

        remote.write_file_to("file", "data")
        p_run_s.assert_called_once_with(ssh_remote._write_file_to, None,
                                        description, "file", "data", False)

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_write_files_to(self, p_log_command, p_run_s):
        instance = FakeInstance('inst8', '123', '10.0.0.8', '10.0.0.8',
                                'user8', 'key8')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'Writing files "[\'file\']"'

        remote.write_files_to({"file": "data"})
        p_run_s.assert_called_once_with(ssh_remote._write_files_to, None,
                                        description, {"file": "data"}, False)

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_append_to_file(self, p_log_command, p_run_s):
        instance = FakeInstance('inst9', '123', '10.0.0.9', '10.0.0.9',
                                'user9', 'key9')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'Appending to file "file"'

        remote.append_to_file("file", "data")
        p_run_s.assert_called_once_with(ssh_remote._append_to_file, None,
                                        description, "file", "data", False)

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_append_to_files(self, p_log_command, p_run_s):
        instance = FakeInstance('inst10', '123',
                                '10.0.0.10', '10.0.0.10', 'user10', 'key10')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'Appending to files "[\'file\']"'

        remote.append_to_files({"file": "data"})
        p_run_s.assert_called_once_with(ssh_remote._append_to_files, None,
                                        description, {"file": "data"}, False)

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_read_file_from(self, p_log_command, p_run_s):
        instance = FakeInstance('inst11', '123',
                                '10.0.0.11', '10.0.0.11', 'user11', 'key11')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'Reading file "file"'

        remote.read_file_from("file")
        p_run_s.assert_called_once_with(ssh_remote._read_file_from, None,
                                        description, "file", False)

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_replace_remote_string(self, p_log_command, p_run_s):
        instance = FakeInstance('inst12', '123',
                                '10.0.0.12', '10.0.0.12', 'user12', 'key12')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'In file "file" replacing string "str1" with "str2"'

        remote.replace_remote_string("file", "str1", "str2")
        p_run_s.assert_called_once_with(ssh_remote._replace_remote_string,
                                        None, description, "file", "str1",
                                                                   "str2")

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_replace_remote_line(self, p_log_command, p_run_s):
        instance = FakeInstance('inst13', '123',
                                '10.0.0.13', '10.0.0.13', 'user13', 'key13')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = ('In file "file" replacing line beginning with string '
                       '"str" with "newline"')

        remote.replace_remote_line("file", "str", "newline")
        p_run_s.assert_called_once_with(ssh_remote._replace_remote_line,
                                        None, description, "file", "str",
                                                                   "newline")

        p_log_command.assert_called_with(description)

    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._run_s')
    @mock.patch('sahara.utils.ssh_remote.InstanceInteropHelper._log_command')
    def test_execute_on_vm_interactive(self, p_log_command, p_run_s):
        instance = FakeInstance('inst14', '123',
                                '10.0.0.14', '10.0.0.14', 'user14', 'key14')
        remote = ssh_remote.InstanceInteropHelper(instance)
        description = 'Executing interactively "factor 42"'

        remote.execute_on_vm_interactive("factor 42", None)
        p_run_s.assert_called_once_with(ssh_remote._execute_on_vm_interactive,
                                        None, description, "factor 42", None)

        p_log_command(description)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.service.api import v10 as api
from sahara.service.validations.edp import job_binary_internal as jb
from sahara.service.validations.edp.job_binary_internal import jb_manager
from sahara.service.validations.edp import job_binary_internal_schema as jbs
from sahara.tests.unit.service.validation import utils as u


class TestJobBinaryInternalCreateValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestJobBinaryInternalCreateValidation, self).setUp()
        self._create_object_fun = jb.check_job_binary_internal
        api.plugin_base.setup_plugins()
        jb_manager.setup_job_binaries()

    def test_job_binary_internal_create(self):
        self._assert_create_object_validation(data=b'text')

        self._assert_create_object_validation(
            data='',
            bad_req_i=(1, "BAD_JOB_BINARY",
                       "Job binary internal data must be a string of length "
                       "greater than zero"))


class TestJobBinaryInternalUpdateValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestJobBinaryInternalUpdateValidation, self).setUp()
        self._create_object_fun = mock.Mock()
        self.scheme = jbs.JOB_BINARY_UPDATE_SCHEMA

    def test_job_binary_internal_update_types(self):
        data = {
            'name': 'jb',
            'is_public': False,
            'is_protected': False
        }
        self._assert_types(data)

    def test_job_binary_internal_update(self):
        self._assert_create_object_validation(data={'name': 'jb'})

        self._assert_create_object_validation(
            data={'id': '1',
                  'is_public': False,
                  'is_protected': False},
            bad_req_i=(1, "VALIDATION_ERROR",
                       "Additional properties are not allowed "
                       "('id' was unexpected)"))



# Copyright (c) 2016 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools

from sahara import exceptions as ex
from sahara.service.validations import acl
from sahara.tests.unit import base


class FakeObject(object):
    def __init__(self, protected, values):
        self.values = values
        self.is_protected = protected
        self.id = "fakeid"

    def to_dict(self):
        return self.values


class TestProtectedValidation(base.SaharaTestCase):
    def test_public_on_protected(self):
        prot = FakeObject(True, {"cat": 1,
                                 "dog": 2})
        values = {"cat": 3, "dog": 2}

        # Should raise because prot.is_protected is True
        with testtools.ExpectedException(ex.UpdateFailedException):
            acl.check_protected_from_update(prot, values)

        # Should not raise because values turns is_protected off
        values["is_protected"] = False
        acl.check_protected_from_update(prot, values)

        # Should be allowed because is_public is the only thing
        # that is potentially changing
        values = {"cat": 1, "dog": 2, "is_public": True}
        acl.check_protected_from_update(prot, values)

        values["cat"] = 3
        # Should raise because we are trying to change cat, too
        with testtools.ExpectedException(ex.UpdateFailedException):
            acl.check_protected_from_update(prot, values)



# Copyright 2015 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""drop progress in JobExecution

Revision ID: 017
Revises: 016
Create Date: 2015-02-25 09:23:04.390388

"""

# revision identifiers, used by Alembic.
revision = '017'
down_revision = '016'

from alembic import op


def upgrade():
    op.drop_column('job_executions', 'progress')



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import xml.dom.minidom as xml

import sahara.exceptions as ex
from sahara.i18n import _
from sahara.utils import xmlutils as x


class OozieWorkflowCreator(object):

    doc = None
    tag_name = "no-op"

    def __init__(self, name):
        self.doc = x.load_xml_document("service/edp/resources/workflow.xml",
                                       strip=True)
        self.tag_name = name

        x.add_child(self.doc, 'action', self.tag_name)

        ok_elem = xml.parseString('<%s to="%s"/>' % ("ok", "end"))
        x.add_element(self.doc, 'action', ok_elem.firstChild)
        error_elem = xml.parseString('<%s to="%s"/>' % ("error", "fail"))
        x.add_element(self.doc, 'action', error_elem.firstChild)

        x.add_text_element_to_tag(self.doc, self.tag_name,
                                  'job-tracker', "${jobTracker}")
        x.add_text_element_to_tag(self.doc, self.tag_name,
                                  'name-node', "${nameNode}")

    def _add_to_prepare_element(self, element, paths):
        if element not in ['delete', 'mkdir']:
            raise ex.NotFoundException(element,
                                       _('"%s" child cannot be '
                                         'added to prepare element'))
        prop = x.get_and_create_if_not_exist(self.doc, self.tag_name,
                                             'prepare')
        for path in paths:
            elem = xml.parseString('<%s path="%s"/>' % (element, path))
            prop.appendChild(elem.firstChild)

    def _add_to_streaming_element(self, element, path):
        if element not in ['mapper', 'reducer']:
            raise ex.NotFoundException(element,
                                       _('"%s" child cannot be added '
                                         'to streaming element'))

        x.get_and_create_if_not_exist(self.doc, self.tag_name,
                                      'streaming')

        x.add_text_element_to_tag(self.doc, 'streaming', element, path)

    def _add_configuration_elements(self, configuration):
        if configuration:
            x.add_properties_to_configuration(self.doc, self.tag_name,
                                              configuration)

    def _add_job_xml_element(self, job_xml):
        if job_xml:
            x.add_text_element_to_tag(self.doc, self.tag_name,
                                      'job-xml', job_xml)

    def _add_files_and_archives(self, files, archives):
        if files:
            x.add_tagged_list(self.doc, self.tag_name, 'file', files)
        if archives:
            x.add_tagged_list(self.doc, self.tag_name, 'archive', archives)

    def get_built_workflow_xml(self):
        return self.doc.toprettyxml(indent="  ")



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_utils import uuidutils
import six
from six.moves.urllib import parse as urlparse

from sahara import conductor as c
from sahara import context
from sahara.plugins import exceptions as ex
from sahara.plugins import utils as u
from sahara.utils import cluster as cluster_utils

conductor = c.API

HBASE_COMMON_LIB_PATH = "/user/sahara-hbase-lib"


def create_hbase_common_lib(r):
    r.execute_command(
        'sudo su - -c "hdfs dfs -mkdir -p %s" hdfs' % (
            HBASE_COMMON_LIB_PATH))
    ret_code, stdout = r.execute_command(
        'hbase classpath')
    if ret_code == 0:
        paths = stdout.split(':')
        for p in paths:
            if p.endswith(".jar"):
                r.execute_command('sudo su - -c "hdfs dfs -put -p %s %s" hdfs'
                                  % (p, HBASE_COMMON_LIB_PATH))
    else:
        raise ex.RequiredServiceMissingException('hbase')


def put_file_to_hdfs(r, file, file_name, path, hdfs_user):
    tmp_file_name = '%s.%s' % (file_name, six.text_type(
        uuidutils.generate_uuid()))
    r.write_file_to('/tmp/%s' % tmp_file_name, file)
    move_from_local(r, '/tmp/%s' % tmp_file_name, path + '/' + file_name,
                    hdfs_user)


def copy_from_local(r, source, target, hdfs_user):
    r.execute_command('sudo su - -c "hdfs dfs -copyFromLocal '
                      '%s %s" %s' % (source, target, hdfs_user))


def move_from_local(r, source, target, hdfs_user):
    # using copyFromLocal followed by rm to address permission issues that
    # arise when image user is not the same as hdfs user (permissions-wise).
    r.execute_command('sudo su - -c "hdfs dfs -copyFromLocal %(source)s '
                      '%(target)s" %(user)s && sudo rm -f %(source)s' %
                      {"source": source, "target": target, "user": hdfs_user})


def create_dir_hadoop1(r, dir_name, hdfs_user):
    r.execute_command(
        'sudo su - -c "hdfs dfs -mkdir %s" %s' % (dir_name, hdfs_user))


def create_dir_hadoop2(r, dir_name, hdfs_user):
    r.execute_command(
        'sudo su - -c "hdfs dfs -mkdir -p %s" %s' % (dir_name, hdfs_user))


def _get_cluster_hosts_information(host, cluster):
    for clust in conductor.cluster_get_all(context.ctx()):
        if clust.id == cluster.id:
            continue

        for i in u.get_instances(clust):
            if i.instance_name == host:
                return cluster_utils.generate_etc_hosts(clust)

    return None


def _is_cluster_configured(cluster, host_info):
    inst = u.get_instances(cluster)[0]
    cat_etc_hosts = 'cat /etc/hosts'
    with inst.remote() as r:
        exit_code, etc_hosts = r.execute_command(cat_etc_hosts)
        return all(host in etc_hosts for host in host_info)


def configure_cluster_for_hdfs(cluster, data_source_url):
    host = urlparse.urlparse(data_source_url).hostname

    etc_hosts_information = _get_cluster_hosts_information(host, cluster)
    if etc_hosts_information is None:
        # Ip address hasn't been resolved, the last chance is for VM itself
        return

    # If the cluster was already configured for this data source
    # there's no need to configure it again
    if _is_cluster_configured(cluster, etc_hosts_information.splitlines()):
        return

    etc_hosts_update = ('/tmp/etc-hosts-update'
                        '.%s' % six.text_type(uuidutils.generate_uuid()))
    tmp_etc_hosts = ('/tmp/etc-hosts'
                     '.%s' % six.text_type(uuidutils.generate_uuid()))
    update_etc_hosts_cmd = (
        'cat %(etc_hosts_update)s /etc/hosts | '
        'sort | uniq > %(tmp_etc_hosts)s && '
        'cat %(tmp_etc_hosts)s > /etc/hosts && '
        'rm -f %(tmp_etc_hosts)s %(etc_hosts_update)s' %
        {'etc_hosts_update': etc_hosts_update, 'tmp_etc_hosts': tmp_etc_hosts})

    for inst in u.get_instances(cluster):
        with inst.remote() as r:
            r.write_file_to(etc_hosts_update, etc_hosts_information)
            r.execute_command(update_etc_hosts_cmd, run_as_root=True)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

from oslo_config import cfg
from oslo_log import log as logging
import six
from stevedore import enabled

from sahara import conductor as cond
from sahara import exceptions as ex
from sahara.i18n import _
from sahara.plugins import labels
from sahara.utils import resources

conductor = cond.API

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


def required(fun):
    return abc.abstractmethod(fun)


def required_with_default(fun):
    return fun


def optional(fun):
    fun.__not_implemented__ = True
    return fun


@six.add_metaclass(abc.ABCMeta)
class PluginInterface(resources.BaseResource):
    __resource_name__ = 'plugin'

    name = 'plugin_interface'

    @required
    def get_title(self):
        """Plugin title

        For example:

            "Vanilla Provisioning"
        """
        pass

    @required_with_default
    def get_description(self):
        """Optional description of the plugin

        This information is targeted to be displayed in UI.
        """
        pass

    def to_dict(self):
        return {
            'name': self.name,
            'title': self.get_title(),
            'description': self.get_description(),
        }


class PluginManager(object):
    def __init__(self):
        self.plugins = {}
        self.default_label_schema = {}
        self._load_cluster_plugins()
        self.label_handler = labels.LabelHandler(self.plugins)

    def _load_cluster_plugins(self):
        config_plugins = CONF.plugins
        extension_manager = enabled.EnabledExtensionManager(
            check_func=lambda ext: ext.name in config_plugins,
            namespace='sahara.cluster.plugins',
            invoke_on_load=True
        )

        for ext in extension_manager.extensions:
            if ext.name in self.plugins:
                raise ex.ConfigurationError(
                    _("Plugin with name '%s' already exists.") % ext.name)
            ext.obj.name = ext.name
            self.plugins[ext.name] = ext.obj
            LOG.info("Plugin {plugin_name} loaded {entry_point}".format(
                     plugin_name=ext.name,
                     entry_point=ext.entry_point_target))

        if len(self.plugins) < len(config_plugins):
            self.loaded_plugins = set(six.iterkeys(self.plugins))
            requested_plugins = set(config_plugins)
            LOG.warning("Plugins couldn't be loaded: %s",
                        ", ".join(requested_plugins - self.loaded_plugins))

    def get_plugins(self, serialized=False):
        if serialized:
            return [self.serialize_plugin(name)
                    for name in PLUGINS.plugins]
        return [self.get_plugin(name) for name in PLUGINS.plugins]

    def get_plugin(self, plugin_name):
        return self.plugins.get(plugin_name)

    def is_plugin_implements(self, plugin_name, fun_name):
        plugin = self.get_plugin(plugin_name)

        fun = getattr(plugin, fun_name)

        if not (fun and callable(fun)):
            return False

        return not hasattr(fun, '__not_implemented__')

    def serialize_plugin(self, plugin_name, version=None):
        plugin = self.get_plugin(plugin_name)
        if plugin:
            res = plugin.as_resource()
            res._info.update(self.label_handler.get_label_full_details(
                plugin_name))
            if version:
                if version in plugin.get_versions():
                    res._info.update(plugin.get_version_details(version))
                else:
                    return None
            return res

    def update_plugin(self, plugin_name, values):
        self.label_handler.update_plugin(plugin_name, values)
        return self.serialize_plugin(plugin_name)

    def validate_plugin_update(self, plugin_name, values):
        return self.label_handler.validate_plugin_update(plugin_name, values)

    def get_plugin_update_validation_jsonschema(self):
        return self.label_handler.get_plugin_update_validation_jsonschema()

    def validate_plugin_labels(self, plugin, version):
        self.label_handler.validate_plugin_labels(plugin, version)


PLUGINS = None


def setup_plugins():
    global PLUGINS
    PLUGINS = PluginManager()



# Copyright (c) 2017 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six.moves.urllib.parse as urlparse

from sahara import exceptions as ex
from sahara.i18n import _
from sahara.service.edp.data_sources.base import DataSourceType
from sahara.service.edp import hdfs_helper as h


class HDFSType(DataSourceType):
    def validate(self, data):
        self._validate_url(data['url'])

    def _validate_url(self, url):
        if len(url) == 0:
            raise ex.InvalidDataException(_("HDFS url must not be empty"))
        url = urlparse.urlparse(url)
        if url.scheme:
            if url.scheme != "hdfs":
                raise ex.InvalidDataException(_("URL scheme must be 'hdfs'"))
            if not url.hostname:
                raise ex.InvalidDataException(
                    _("HDFS url is incorrect, cannot determine a hostname"))

    def prepare_cluster(self, data_source, cluster, **kwargs):
        runtime_url = kwargs.pop('runtime_url')
        h.configure_cluster_for_hdfs(cluster, runtime_url)



# Copyright (c) 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_policy import policy as cpolicy

from sahara.api import acl
from sahara import exceptions as ex
from sahara.tests.unit import base


class TestAcl(base.SaharaTestCase):

    def _set_policy(self, json):
        acl.setup_policy()
        rules = cpolicy.Rules.load_json(json)
        acl.ENFORCER.set_rules(rules, use_conf=False)

    @mock.patch('oslo_config.cfg.ConfigOpts.find_file')
    def test_policy_allow(self, mock_config):
        mock_config.return_value = '/etc/sahara/'
        @acl.enforce("data-processing:clusters:get_all")
        def test():
            pass

        json = '{"data-processing:clusters:get_all": ""}'
        self._set_policy(json)

        test()

    @mock.patch('oslo_config.cfg.ConfigOpts.find_file')
    def test_policy_deny(self, mock_config):
        mock_config.return_value = '/etc/sahara/'
        @acl.enforce("data-processing:clusters:get_all")
        def test():
            pass

        json = '{"data-processing:clusters:get_all": "!"}'
        self._set_policy(json)

        self.assertRaises(ex.Forbidden, test)



#!/usr/bin/env python

# Copyright (c) 2012, AT&T Labs, Yun Mao <yunmao@gmail.com>
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Stolen from OpenStack Nova

"""pylint error checking."""
import json
import re
import sys

from pylint import lint
from pylint.reporters import text
from six.moves import cStringIO as StringIO

# Note(maoy): E1103 is error code related to partial type inference
ignore_codes = ["E1103"]
# Note(maoy): the error message is the pattern of E0202. It should be ignored
# for sahara.tests modules
ignore_messages = ["An attribute affected in sahara.tests"]

KNOWN_PYLINT_EXCEPTIONS_FILE = "tools/pylint_exceptions"


class LintOutput(object):

    _cached_filename = None
    _cached_content = None

    def __init__(self, filename, lineno, line_content, code, message,
                 lintoutput):
        self.filename = filename
        self.lineno = lineno
        self.line_content = line_content
        self.code = code
        self.message = message
        self.lintoutput = lintoutput

    @classmethod
    def from_line(cls, line):
        m = re.search(r"(\S+):(\d+): \[(\S+)(, \S*)?] (.*)", line)
        matched = m.groups()
        filename, lineno, code, message = (matched[0], int(matched[1]),
                                           matched[2], matched[-1])
        if cls._cached_filename != filename:
            with open(filename) as f:
                cls._cached_content = list(f.readlines())
                cls._cached_filename = filename
        line_content = cls._cached_content[lineno - 1].rstrip()
        return cls(filename, lineno, line_content, code, message,
                   line.rstrip())

    @classmethod
    def from_msg_to_dict(cls, msg):
        """From the output of pylint msg, to a dict, where each key
        is a unique error identifier, value is a list of LintOutput
        """
        result = {}
        for line in msg.splitlines():
            if line.startswith('*****'):
                continue
            obj = cls.from_line(line)
            if obj.is_ignored():
                continue
            key = obj.key()
            if key not in result:
                result[key] = []
            result[key].append(obj)
        return result

    def is_ignored(self):
        if self.code in ignore_codes:
            return True
        if any(msg in self.message for msg in ignore_messages):
            return True
        return False

    def key(self):
        if self.code in ["E1101", "E1103"]:
            # These two types of errors are like Foo class has no member bar.
            # We discard the source code so that the error will be ignored
            # next time another Foo.bar is encountered.
            return self.message, ""
        return self.message, self.line_content.strip()

    def json(self):
        return json.dumps(self.__dict__)

    def review_str(self):
        return ("File %(filename)s\nLine %(lineno)d:%(line_content)s\n"
                "%(code)s: %(message)s" % self.__dict__)


class ErrorKeys(object):

    @classmethod
    def print_json(cls, errors, output=sys.stdout):
        print("# automatically generated by tools/lintstack.py", file=output)
        for i in sorted(errors.keys()):
            print(json.dumps(i), file=output)

    @classmethod
    def from_file(cls, filename):
        keys = set()
        for line in open(filename):
            if line and line[0] != "#":
                d = json.loads(line)
                keys.add(tuple(d))
        return keys


def run_pylint():
    buff = StringIO()
    reporter = text.ParseableTextReporter(output=buff)
    args = ["--include-ids=y", "-E", "sahara"]
    lint.Run(args, reporter=reporter, exit=False)
    val = buff.getvalue()
    buff.close()
    return val


# noinspection PyTypeChecker
def generate_error_keys(msg=None):
    print("Generating", KNOWN_PYLINT_EXCEPTIONS_FILE)
    if msg is None:
        msg = run_pylint()
    errors = LintOutput.from_msg_to_dict(msg)
    with open(KNOWN_PYLINT_EXCEPTIONS_FILE, "w") as f:
        ErrorKeys.print_json(errors, output=f)


def validate(newmsg=None):
    print("Loading", KNOWN_PYLINT_EXCEPTIONS_FILE)
    known = ErrorKeys.from_file(KNOWN_PYLINT_EXCEPTIONS_FILE)
    if newmsg is None:
        print("Running pylint. Be patient...")
        newmsg = run_pylint()
    errors = LintOutput.from_msg_to_dict(newmsg)

    print("Unique errors reported by pylint: was %d, now %d."
          % (len(known), len(errors)))
    passed = True
    for err_key, err_list in errors.items():
        for err in err_list:
            if err_key not in known:
                print(err.lintoutput)
                print()
                passed = False
    if passed:
        print("Congrats! pylint check passed.")
        redundant = known - set(errors.keys())
        if redundant:
            print("Extra credit: some known pylint exceptions disappeared.")
            for i in sorted(redundant):
                print(json.dumps(i))
            print("Consider regenerating the exception file if you will.")
    else:
        print("Please fix the errors above. If you believe they are false"
              " positives, run 'tools/lintstack.py generate' to overwrite.")
        sys.exit(1)


def usage():
    print("""Usage: tools/lintstack.py [generate|validate]
    To generate pylint_exceptions file: tools/lintstack.py generate
    To validate the current commit: tools/lintstack.py
    """)


def main():
    option = "validate"
    if len(sys.argv) > 1:
        option = sys.argv[1]
    if option == "generate":
        generate_error_keys()
    elif option == "validate":
        validate()
    else:
        usage()


if __name__ == "__main__":
    main()



# Copyright (c) 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


try:
    from manilaclient.common.apiclient import exceptions as manila_ex
except ImportError:
    from manilaclient.openstack.common.apiclient import exceptions as manila_ex
from unittest import mock

from oslo_utils import uuidutils
import testtools

from sahara import exceptions
from sahara.service.edp.utils import shares
from sahara.tests.unit import base

_NAMENODE_IPS = ['192.168.122.3', '192.168.122.4']
_DATANODE_IPS = ['192.168.122.5', '192.168.122.6', '192.168.122.7']


class _FakeShare(object):

    def __init__(self, id='12345678-1234-1234-1234-123456789012',
                 share_proto='NFS',
                 export_location='192.168.122.1:/path',
                 access_list=None):
        self.id = id
        self.share_proto = share_proto
        self.export_location = export_location
        self.allow = mock.Mock()
        self.deny = mock.Mock()
        self.access_list = mock.Mock(return_value=access_list or [])


def _mock_node_group(ips, share_list):
    # Returns a mocked node group and a list of mocked
    # execute_command functions for its instances.

    execute_mocks = [mock.Mock(return_value="centos") for ip in ips]
    get_id = mock.Mock(return_value=uuidutils.generate_uuid())
    instances = [
        mock.Mock(
            internal_ip=ip,
            remote=mock.Mock(
                return_value=mock.Mock(
                    __enter__=mock.Mock(
                        return_value=mock.Mock(
                            execute_command=execute_mocks[index],
                            get_os_distrib=execute_mocks[index])),
                    __exit__=mock.Mock())))
        for index, ip in enumerate(ips)]

    node_group = mock.Mock(instances=instances,
                           shares=share_list,
                           __getitem__=get_id)
    return node_group, execute_mocks


def _setup_calls():
    return [
        mock.call('rpm -q nfs-utils || yum install -y nfs-utils',
                  run_as_root=True)]


def _expected_calls(local_path, remote_path, access_argument):
    return [
        mock.call('mkdir -p %s' % local_path, run_as_root=True),
        mock.call("mount | grep '%(remote_path)s' | grep '%(local_path)s' | "
                  "grep nfs || mount -t nfs %(access_argument)s "
                  "%(remote_path)s %(local_path)s" %
                  {
                      "local_path": local_path,
                      "remote_path": remote_path,
                      "access_argument": access_argument
                  },
                  run_as_root=True)]


class TestShares(base.SaharaTestCase):

    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_mount_nfs_shares_to_ng(self, f_manilaclient, f_context):

        share = _FakeShare()
        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(return_value=share)))

        namenode_group, namenode_executors = _mock_node_group(
            _NAMENODE_IPS,
            [{
                'id': '12345678-1234-1234-1234-123456789012',
                'access_level': 'rw',
                'path': '/mnt/localpath'
            }])

        datanode_group, datanode_executors = _mock_node_group(
            _DATANODE_IPS, [])

        cluster = mock.Mock(
            node_groups=[namenode_group, datanode_group], shares=[])

        shares.mount_shares(cluster)

        permissions = [mock.call('ip', ip, 'rw') for ip in _NAMENODE_IPS]
        share.allow.assert_has_calls(permissions, any_order=True)

        for executor in namenode_executors:
            executor.assert_has_calls(
                _setup_calls() +
                _expected_calls('/mnt/localpath', '192.168.122.1:/path', '-w'))
        for executor in datanode_executors:
            self.assertEqual(0, executor.call_count)

    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_mount_nfs_shares_to_cluster(self, f_manilaclient, f_context):

        global_share = _FakeShare()
        namenode_only_share = _FakeShare(
            id='DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF',
            export_location='192.168.122.2:/path')
        all_shares = {share.id: share for share in
                      (global_share, namenode_only_share)}

        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(
                    side_effect=lambda x: all_shares[x])))

        namenode_group, namenode_executors = _mock_node_group(
            ['192.168.122.3', '192.168.122.4'],
            [
                {
                    'id': '12345678-1234-1234-1234-123456789012',
                    'access_level': 'rw',
                    'path': '/mnt/localpath'
                },
                {
                    'id': 'DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF'
                }
            ])

        datanode_group, datanode_executors = _mock_node_group(
            ['192.168.122.5', '192.168.122.6', '192.168.122.7'], [])

        cluster = mock.Mock(
            node_groups=[namenode_group, datanode_group],
            shares=[
                {
                    'id': '12345678-1234-1234-1234-123456789012',
                    'access_level': 'ro',
                    'path': '/mnt/somanylocalpaths'
                }
            ])

        shares.mount_shares(cluster)

        all_permissions = [mock.call('ip', ip, 'ro')
                           for ip in _NAMENODE_IPS + _DATANODE_IPS]
        global_share.allow.assert_has_calls(all_permissions, any_order=True)

        namenode_permissions = [mock.call('ip', ip, 'rw')
                                for ip in _NAMENODE_IPS]
        namenode_only_share.allow.assert_has_calls(namenode_permissions,
                                                   any_order=True)

        for executor in namenode_executors:
            executor.assert_has_calls(
                _setup_calls() +
                _expected_calls('/mnt/somanylocalpaths',
                                '192.168.122.1:/path', '-r') +
                _expected_calls('/mnt/DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF',
                                '192.168.122.2:/path', '-w'),
                any_order=True)
            self.assertEqual(6, executor.call_count)

        for executor in datanode_executors:
            executor.assert_has_calls(
                _setup_calls() +
                _expected_calls('/mnt/somanylocalpaths',
                                '192.168.122.1:/path', '-r'))
            self.assertEqual(4, executor.call_count)

    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_share_does_not_exist(self, f_manilaclient, f_context):

        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(
                    side_effect=manila_ex.NotFound)))

        namenode_group, namenode_executors = _mock_node_group(
            ['192.168.122.3', '192.168.122.4'],
            [
                {
                    'id': '12345678-1234-1234-1234-123456789012',
                    'access_level': 'rw',
                    'path': '/mnt/localpath'
                },
                {
                    'id': 'DEADBEEF-DEAD-BEEF-DEAD-BEEFDEADBEEF'
                }
            ])

        datanode_group, datanode_executors = _mock_node_group(
            ['192.168.122.5', '192.168.122.6', '192.168.122.7'], [])

        cluster = mock.Mock(
            node_groups=[namenode_group, datanode_group],
            shares=[
                {
                    'id': '12345678-1234-1234-1234-123456789012',
                    'access_level': 'ro',
                    'path': '/mnt/somanylocalpaths'
                }
            ])

        with testtools.ExpectedException(exceptions.NotFoundException):
            shares.mount_shares(cluster)

    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_acl_exists_unexpected_type(self, f_manilaclient, f_context):

        share = _FakeShare(access_list=[mock.Mock(
            access_level='wat', access_to=ip, access_type='ip')
            for ip in _NAMENODE_IPS])
        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(return_value=share)))

        namenode_group, namenode_executors = _mock_node_group(
            _NAMENODE_IPS,
            [{
                'id': '12345678-1234-1234-1234-123456789012',
                'access_level': 'rw',
                'path': '/mnt/localpath'
            }])

        datanode_group, datanode_executors = _mock_node_group(
            _DATANODE_IPS, [])

        cluster = mock.Mock(
            node_groups=[namenode_group, datanode_group], shares=[])

        shares.mount_shares(cluster)

        self.assertEqual(0, share.allow.call_count)

        for executor in namenode_executors:
            executor.assert_has_calls(
                _setup_calls() +
                _expected_calls('/mnt/localpath', '192.168.122.1:/path', '-w'))

        for executor in datanode_executors:
            self.assertEqual(0, executor.call_count)

    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_acl_exists_no_recreate(self, f_manilaclient, f_context):

        share = _FakeShare(access_list=[mock.Mock(
            access_level='rw', access_to=ip, access_type='ip')
            for ip in _NAMENODE_IPS])
        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(return_value=share)))

        namenode_group, namenode_executors = _mock_node_group(
            _NAMENODE_IPS,
            [{
                'id': '12345678-1234-1234-1234-123456789012',
                'access_level': 'ro',
                'path': '/mnt/localpath'
            }])

        datanode_group, datanode_executors = _mock_node_group(
            _DATANODE_IPS, [])

        cluster = mock.Mock(
            node_groups=[namenode_group, datanode_group], shares=[])

        shares.mount_shares(cluster)

        self.assertEqual(0, share.allow.call_count)

        for executor in namenode_executors:
            executor.assert_has_calls(
                _setup_calls() +
                _expected_calls('/mnt/localpath', '192.168.122.1:/path', '-r'))

        for executor in datanode_executors:
            self.assertEqual(0, executor.call_count)

    @mock.patch('sahara.context.set_current_instance_id')
    @mock.patch('sahara.utils.openstack.manila.client')
    def test_acl_exists_recreate(self, f_manilaclient, f_context):

        share = _FakeShare(access_list=[mock.Mock(
            access_level='ro', access_to=ip, access_type='ip', id="access_id")
            for ip in _NAMENODE_IPS])
        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(return_value=share)))

        namenode_group, namenode_executors = _mock_node_group(
            _NAMENODE_IPS,
            [{
                'id': '12345678-1234-1234-1234-123456789012',
                'access_level': 'rw',
                'path': '/mnt/localpath'
            }])

        datanode_group, datanode_executors = _mock_node_group(
            _DATANODE_IPS, [])

        cluster = mock.Mock(
            node_groups=[namenode_group, datanode_group], shares=[])

        shares.mount_shares(cluster)

        namenode_denials = [mock.call('access_id')
                            for ip in _NAMENODE_IPS]
        share.deny.assert_has_calls(namenode_denials)

        namenode_permissions = [mock.call('ip', ip, 'rw')
                                for ip in _NAMENODE_IPS]
        share.allow.assert_has_calls(namenode_permissions,
                                     any_order=True)

        for executor in namenode_executors:
            executor.assert_has_calls(
                _setup_calls() +
                _expected_calls('/mnt/localpath', '192.168.122.1:/path', '-w'))
        for executor in datanode_executors:
            self.assertEqual(0, executor.call_count)

    def test_get_share_path(self):
        share_list = [
            {'id': 'the_share_id',
             'path': '/mnt/mymountpoint'},
            {'id': 'the_share_id',
             'path': '/mnt/othermountpoint'},
            {'id': '123456',
             'path': '/mnt/themountpoint'}
        ]
        url = 'manila://the_share_id/the_path'

        path = shares.get_share_path(url, share_list)
        self.assertEqual("/mnt/mymountpoint/the_path", path)

        share_list.pop(0)
        path = shares.get_share_path(url, share_list)
        self.assertEqual("/mnt/othermountpoint/the_path", path)

        share_list.pop(0)
        path = shares.get_share_path(url, share_list)
        self.assertIsNone(path)

    @mock.patch('sahara.utils.openstack.manila.client')
    def test_get_share_path_default(self, f_manilaclient):
        share_list = [
            {'id': 'i_have_no_mnt'}
        ]

        share = _FakeShare(share_list[0]['id'])
        f_manilaclient.return_value = mock.Mock(
            shares=mock.Mock(
                get=mock.Mock(return_value=share)))

        url = 'manila://i_have_no_mnt/the_path'
        path = shares.get_share_path(url, share_list)
        self.assertEqual("/mnt/i_have_no_mnt/the_path", path)



# Copyright (c) 2015 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from unittest import mock


import sahara.exceptions as ex
from sahara.service.edp.job_utils import ds_manager
from sahara.service.edp.storm import engine as se
from sahara.service.edp.storm.engine import jb_manager
from sahara.tests.unit import base
from sahara.utils import edp


class TestStorm(base.SaharaTestCase):
    def setUp(self):
        super(TestStorm, self).setUp()

        self.master_host = "master"
        self.master_inst = "6789"
        self.storm_topology_name = "MyJob_ed8347a9-39aa-477c-8108-066202eb6130"
        self.workflow_dir = "/wfdir"

        jb_manager.setup_job_binaries()
        ds_manager.setup_data_sources()

    def test_get_topology_and_inst_id(self):
        '''Test parsing of job ids

        Test that job ids of the form topology_name@instance are
        split into topology_name and instance ids by
        eng._get_topology_name_and_inst_id() but anything else
        returns empty strings
        '''
        eng = se.StormJobEngine(None)
        for job_id in [None, "", "@", "something", "topology_name@",
                       "@instance"]:
            topology_name, inst_id = eng._get_topology_and_inst_id(job_id)
            self.assertEqual(("", ""), (topology_name, inst_id))

        topology_name, inst_id = eng._get_topology_and_inst_id(
            "topology_name@instance")
        self.assertEqual(("topology_name", "instance"),
                         (topology_name, inst_id))

    @mock.patch('sahara.utils.cluster.get_instances')
    def test_get_instance_if_running(self, get_instances):
        '''Test retrieval of topology_name and instance object for running job

        If the job id is valid and the job status is non-terminated,
        _get_instance_if_running() should retrieve the instance
        based on the inst_id and return the topology_name and instance.

        If the job is invalid or the job is terminated, it should
        return None, None.

        If get_instances() throws an exception or returns an empty list,
        the instance returned should be None (topology_name might
        still be set)
        '''
        get_instances.return_value = ["instance"]
        job_exec = mock.Mock()
        eng = se.StormJobEngine("cluster")

        job_exec.engine_job_id = "invalid id"
        self.assertEqual((None, None),
                         eng._get_instance_if_running(job_exec))

        job_exec.engine_job_id = "topology_name@inst_id"
        for state in edp.JOB_STATUSES_TERMINATED:
            job_exec.info = {'status': state}
            self.assertEqual((None, None),
                             eng._get_instance_if_running(job_exec))

        job_exec.info = {'status': edp.JOB_STATUS_RUNNING}
        self.assertEqual(("topology_name", "instance"),
                         eng._get_instance_if_running(job_exec))
        get_instances.assert_called_with("cluster", ["inst_id"])

        # Pretend get_instances returns nothing
        get_instances.return_value = []
        topology_name, instance = eng._get_instance_if_running(job_exec)
        self.assertIsNone(instance)

        # Pretend get_instances throws an exception
        get_instances.side_effect = Exception("some failure")
        topology_name, instance = eng._get_instance_if_running(job_exec)
        self.assertIsNone(instance)

    @mock.patch('sahara.plugins.utils.get_instance')
    @mock.patch('sahara.utils.cluster.get_instances')
    @mock.patch('sahara.utils.remote.get_remote')
    @mock.patch('sahara.conductor.API.job_get')
    @mock.patch('sahara.context.ctx', return_value="ctx")
    def test_get_job_status_from_remote(self, get_instance, get_instances,
                                        get_remote, ctx, job_get):
        '''Test retrieval of job status from remote instance

        If the process is present, status is RUNNING
        If the process is not present, status depends on the result file
        If the result file is missing, status is DONEWITHERROR
        '''
        eng = se.StormJobEngine("cluster")
        job_exec = mock.Mock()

        master_instance = self._make_master_instance()
        master_instance.execute_command.return_value = 0, "ACTIVE"
        get_remote.return_value.__enter__ = mock.Mock(
            return_value=master_instance)
        get_instance.return_value = master_instance
        get_instances.return_value = ["instance"]

        # Pretend process is running
        job_exec.engine_job_id = "topology_name@inst_id"
        job_exec.info = {'status': edp.JOB_STATUS_RUNNING}
        job_exec.job_configs = {"configs": {"topology_name": "topology_name"}}
        status = eng._get_job_status_from_remote(job_exec)
        self.assertEqual({"status": edp.JOB_STATUS_RUNNING}, status)

    @mock.patch.object(se.StormJobEngine,
                       '_get_job_status_from_remote',
                       autospec=True)
    @mock.patch.object(se.StormJobEngine,
                       '_get_instance_if_running',
                       autospec=True)
    @mock.patch('sahara.utils.remote.get_remote')
    def test_get_job_status(self,
                            get_remote,
                            _get_instance_if_running,
                            _get_job_status_from_remote):

        # This is to mock "with remote.get_remote(instance) as r"
        remote_instance = mock.Mock()
        get_remote.return_value.__enter__ = mock.Mock(
            return_value=remote_instance)

        # Pretend instance is not returned
        _get_instance_if_running.return_value = "topology_name", None
        job_exec = mock.Mock()
        eng = se.StormJobEngine("cluster")
        status = eng.get_job_status(job_exec)
        self.assertIsNone(status)

        # Pretend we have an instance
        _get_instance_if_running.return_value = "topology_name", "instance"
        _get_job_status_from_remote.return_value = {"status":
                                                    edp.JOB_STATUS_RUNNING}
        status = eng.get_job_status(job_exec)
        _get_job_status_from_remote.assert_called_with(eng,
                                                       job_exec, 3)
        self.assertEqual({"status": edp.JOB_STATUS_RUNNING}, status)

    @mock.patch.object(se.StormJobEngine,
                       '_get_instance_if_running',
                       autospec=True,
                       return_value=(None, None))
    @mock.patch('sahara.utils.remote.get_remote')
    @mock.patch('sahara.conductor.API.job_get')
    @mock.patch('sahara.context.ctx', return_value="ctx")
    def test_cancel_job_null_or_done(self,
                                     get_remote,
                                     _get_instance_if_running,
                                     job_get,
                                     ctx):
        '''Test cancel_job() when instance is None

        Test that cancel_job() returns None and does not try to
        retrieve a remote instance if _get_instance_if_running() returns None
        '''
        eng = se.StormJobEngine("cluster")
        job_exec = mock.Mock()
        self.assertIsNone(eng.cancel_job(job_exec))
        self.assertFalse(get_remote.called)

    @mock.patch.object(se.StormJobEngine,
                       '_get_job_status_from_remote',
                       autospec=True,
                       return_value={"status": edp.JOB_STATUS_KILLED})
    @mock.patch('sahara.utils.cluster.get_instances')
    @mock.patch('sahara.plugins.utils.get_instance')
    @mock.patch('sahara.utils.remote.get_remote')
    def test_cancel_job(self, get_remote, get_instance, get_instances,
                        _get_job_status_from_remote):
        master_instance = self._make_master_instance()
        status = self._setup_tests(master_instance)
        get_instance.return_value = master_instance
        get_instances.return_value = ["instance"]
        master_instance.execute_command.return_value = 0, "KILLED"

        get_remote.return_value.__enter__ = mock.Mock(
            return_value=master_instance)
        eng = se.StormJobEngine("cluster")
        job_exec = mock.Mock()
        job_exec.engine_job_id = "topology_name@inst_id"
        job_exec.info = {'status': edp.JOB_STATUS_RUNNING}
        job_exec.job_configs = {"configs": {"topology_name": "topology_name"}}
        status = eng.cancel_job(job_exec)

        master_instance.execute_command.assert_called_with(
            "/usr/local/storm/bin/storm kill -c nimbus.host=%s topology_name "
            "> /dev/null 2>&1 & echo $!" % self.master_host)

        self.assertEqual({"status": edp.JOB_STATUS_KILLED}, status)

    @mock.patch('sahara.service.edp.storm.engine.jb_manager')
    @mock.patch('sahara.utils.remote.get_remote')
    def test_upload_job_files(self, get_remote, jb_manager):
        main_names = ["main1", "main2", "main3"]
        lib_names = ["lib1", "lib2", "lib3"]

        def make_data_objects(*args):
            objs = []
            for name in args:
                m = mock.Mock()
                m.name = name
                objs.append(m)
            return objs

        job = mock.Mock()
        job.id = "job_exec_id"
        job.mains = make_data_objects(*main_names)
        job.libs = make_data_objects(*lib_names)

        # This is to mock "with remote.get_remote(instance) as r"
        remote_instance = mock.Mock()
        get_remote.return_value.__enter__ = mock.Mock(
            return_value=remote_instance)
        remote_instance.instance.node_group.cluster.shares = []
        remote_instance.instance.node_group.shares = []

        JOB_BINARIES = mock.Mock()
        mock_jb = mock.Mock()
        jb_manager.JOB_BINARIES = JOB_BINARIES

        JOB_BINARIES.get_job_binary_by_url = mock.Mock(return_value=mock_jb)

        jbs = main_names + lib_names

        mock_jb.copy_binary_to_cluster = mock.Mock(
            side_effect=['/tmp/%s.%s' % (job.id, j) for j in jbs])

        eng = se.StormJobEngine("cluster")
        eng._prepare_job_binaries = mock.Mock()

        paths = eng._upload_job_files("where", "/somedir", job, {})
        self.assertEqual(['/tmp/%s.%s' % (job.id, j) for j in jbs],
                         paths)

    def _make_master_instance(self, return_code=0):
        master = mock.Mock()
        master.execute_command.return_value = (return_code,
                                               self.storm_topology_name)
        master.get_python_version.return_value = 'python'
        master.hostname.return_value = self.master_host
        master.id = self.master_inst
        return master

    @mock.patch('sahara.conductor.API.job_execution_get')
    @mock.patch('sahara.utils.remote.get_remote')
    @mock.patch('sahara.plugins.utils.get_instance')
    @mock.patch('sahara.conductor.API.job_get')
    @mock.patch('sahara.context.ctx', return_value="ctx")
    def _setup_tests(self, master_instance, ctx, job_get,
                     get_instance, get_remote, job_exec_get):

        # This is to mock "with remote.get_remote(master) as r" in run_job
        get_remote.return_value.__enter__ = mock.Mock(
            return_value=master_instance)
        get_instance.return_value = master_instance

    @mock.patch.object(se.StormJobEngine,
                       '_generate_topology_name',
                       autospec=True,
                       return_value=(
                           "MyJob_ed8347a9-39aa-477c-8108-066202eb6130"))
    @mock.patch('sahara.conductor.API.job_execution_update')
    @mock.patch('sahara.conductor.API.job_execution_get')
    @mock.patch('sahara.utils.remote.get_remote')
    @mock.patch('sahara.service.edp.job_utils.create_workflow_dir')
    @mock.patch('sahara.plugins.utils.get_instance')
    @mock.patch('sahara.conductor.API.job_get')
    @mock.patch('sahara.context.ctx', return_value="ctx")
    def _setup_run_job(self, master_instance, job_configs, files,
                       ctx, job_get, get_instance, create_workflow_dir,
                       get_remote, job_exec_get, job_exec_update,
                       _generate_topology_name):

        def _upload_job_files(where, job_dir, job,
                              libs_subdir=True, job_configs=None):
            paths = [os.path.join(self.workflow_dir, f) for f in files['jars']]
            return paths

        job = mock.Mock()
        job.name = "MyJob"
        job_get.return_value = job

        job_exec = mock.Mock()
        job_exec.job_configs = job_configs

        create_workflow_dir.return_value = self.workflow_dir

        # This is to mock "with remote.get_remote(master) as r" in run_job
        get_remote.return_value.__enter__ = mock.Mock(
            return_value=master_instance)
        get_instance.return_value = master_instance

        eng = se.StormJobEngine("cluster")
        eng._upload_job_files = mock.Mock()
        eng._upload_job_files.side_effect = _upload_job_files
        status = eng.run_job(job_exec)

        # Check that we launch on the master node
        get_instance.assert_called_with("cluster", "nimbus")

        return status

    def test_run_job_raise(self):
        job_configs = {
            'configs': {"edp.java.main_class": "org.me.myclass",
                        "topology_name": "topology_name"},
        }

        files = {'jars': ["app.jar"]}

        # The object representing the storm master node
        # The storm jar command will be run on this instance
        master_instance = self._make_master_instance(return_code=1)

        # If execute_command returns an error we should get a raise
        self.assertRaises(ex.EDPError,
                          self._setup_run_job,
                          master_instance, job_configs, files)

    def test_run_job(self):
        job_configs = {
            'configs': {"edp.java.main_class": "org.me.myclass"}
        }

        files = {'jars': ["app.jar"]}

        # The object representing the storm master node
        # The storm jar command will be run on this instance
        master_instance = self._make_master_instance()
        status = self._setup_run_job(master_instance, job_configs, files)

        # Check the command
        master_instance.execute_command.assert_called_with(
            'cd %(workflow_dir)s; '
            './launch_command /usr/local/storm/bin/storm jar '
            '-c nimbus.host=master '
            '%(workflow_dir)s/app.jar org.me.myclass %(topology_name)s '
            '> /dev/null 2>&1 & echo $!' % {"workflow_dir": self.workflow_dir,
                                            "topology_name": (
                                                self.storm_topology_name)})

        # Check result here
        self.assertEqual(("%s@%s" % (self.storm_topology_name,
                                     self.master_inst),
                          edp.JOB_STATUS_RUNNING,
                          {"storm-path": self.workflow_dir}), status)

    def test_run_job_args(self):
        job_configs = {
            'configs': {"edp.java.main_class": "org.me.myclass"},
            'args': ['input_arg', 'output_arg']
        }

        files = {'jars': ["app.jar"]}

        # The object representing the spark master node
        # The spark-submit command will be run on this instance
        master_instance = self._make_master_instance()
        status = self._setup_run_job(master_instance, job_configs, files)

        # Check the command
        master_instance.execute_command.assert_called_with(
            'cd %(workflow_dir)s; '
            './launch_command /usr/local/storm/bin/storm jar '
            '-c nimbus.host=master '
            '%(workflow_dir)s/app.jar org.me.myclass %(topology_name)s '
            'input_arg output_arg '
            '> /dev/null 2>&1 & echo $!' % {"workflow_dir": self.workflow_dir,
                                            "topology_name": (
                                                self.storm_topology_name)})

        # Check result here
        self.assertEqual(("%s@%s" % (self.storm_topology_name,
                                     self.master_inst),
                          edp.JOB_STATUS_RUNNING,
                          {"storm-path": self.workflow_dir}), status)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

import six
import testtools

from sahara import exceptions as ex
from sahara.plugins.fake import plugin
from sahara.service.api import v10 as api
import sahara.service.validation as v
from sahara.service.validations import clusters_scaling as c_s
from sahara.service.validations import clusters_schema as c_schema
from sahara.tests.unit.service.validation import utils as u
from sahara.tests.unit import testutils as tu
from sahara.utils import cluster as c_u


def _get_plugin(plugin_name):
    if plugin_name == 'fake':
        return plugin.FakePluginProvider()
    return None


class TestScalingValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestScalingValidation, self).setUp()
        api.plugin_base.setup_plugins()
        self._create_object_fun = mock.Mock()
        self.duplicates_detected = ("Duplicates in node group names are"
                                    " detected: ['a']")
        self.setup_context(tenant_id='tenant1')

    @mock.patch('sahara.service.api.v10.get_cluster')
    @mock.patch('sahara.plugins.base.PluginManager.get_plugin')
    def _assert_check_scaling(self,
                              get_plugin_p=None,
                              get_cluster_p=None,
                              data=None, cluster=None,
                              expected_message=None,
                              expected_exception=ex.InvalidReferenceException):

        get_cluster_p.return_value = cluster
        get_plugin_p.side_effect = _get_plugin

        with testtools.ExpectedException(expected_exception):
            try:
                c_s.check_cluster_scaling(data, cluster.id)
            except expected_exception as e:
                message = six.text_type(e).split('\n')[0]
                self.assertEqual(expected_message, message)
                raise e

    @mock.patch('sahara.service.api.v10.get_node_group_template')
    @mock.patch('sahara.utils.openstack.nova.client')
    @mock.patch("sahara.service.api.OPS")
    def test_check_cluster_scaling_resize_ng(self, ops, nova_client, get_ngt):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        ng1 = tu.make_ng_dict('ng', '42', ['namenode'], 1,
                              node_group_template_id='aaa')
        ng2 = tu.make_ng_dict('ng', '42', ['namenode'], 1,
                              resource=True)
        cluster = tu.create_cluster("cluster1", "tenant1", "fake", "0.1",
                                    [ng1],
                                    status=c_u.CLUSTER_STATUS_VALIDATING,
                                    id='12321')

        self._assert_check_scaling(
            data={}, cluster=cluster,
            expected_message="Cluster cannot be scaled "
                             "not in '" + c_u.CLUSTER_STATUS_ACTIVE +
            "' status. Cluster status: " +
            c_u.CLUSTER_STATUS_VALIDATING)

        cluster = tu.create_cluster("cluster1", "tenant1", "fake", "0.1",
                                    [ng1], status=c_u.CLUSTER_STATUS_ACTIVE,
                                    id='12321')
        data = {
            'resize_node_groups': [
                {
                    'name': 'a',
                    'count': 2
                }
            ],
        }
        self._assert_check_scaling(
            data=data, cluster=cluster,
            expected_message="Cluster doesn't contain "
                             "node group with name 'a'")
        data.update({'resize_node_groups': [
            {
                'name': 'a',
                'count': 2
            },
            {
                'name': 'a',
                'count': 2
            }
        ]})
        self._assert_check_scaling(
            data=data, cluster=cluster,
            expected_message=self.duplicates_detected,
            expected_exception=ex.InvalidDataException)

        data = {
            'resize_node_groups': [
                {
                    'name': 'ng',
                    'count': 2
                }
            ],
        }

        client = mock.Mock()
        nova_client.return_value = client
        client.flavors.list.return_value = []
        get_ngt.return_value = ng2

        self._assert_check_scaling(
            data=data, cluster=cluster,
            expected_message="Requested flavor '42' not found",
            expected_exception=ex.NotFoundException)

    @mock.patch("sahara.service.api.OPS")
    def test_check_cluster_scaling_add_ng(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        ng1 = tu.make_ng_dict('ng', '42', ['namenode'], 1)
        cluster = tu.create_cluster("test-cluster", "tenant1", "fake",
                                    "0.1",
                                    [ng1], status=c_u.CLUSTER_STATUS_ACTIVE,
                                    id='12321')
        data = {
            'add_node_groups': [
                {
                    'name': 'a',
                    'flavor_id': '42',
                    'node_processes': ['namenode']
                },
                {
                    'name': 'a',
                    'flavor_id': '42',
                    'node_processes': ['namenode']
                }
            ]
        }
        self._assert_check_scaling(
            data=data, cluster=cluster,
            expected_message=self.duplicates_detected,
            expected_exception=ex.InvalidDataException)
        data = {
            'add_node_groups': [
                {
                    'name': 'ng',
                    'flavor_id': '42',
                    'node_processes': ['namenode']
                },
            ]
        }
        self._assert_check_scaling(
            data=data, cluster=cluster,
            expected_message="Can't add new nodegroup. "
                             "Cluster already has nodegroup "
                             "with name 'ng'")

        data = {
            'add_node_groups': [
                {
                    'name': 'very-very-very-very-very-very-long-ng-name',
                    'flavor_id': '42',
                    'node_processes': ['namenode'],
                    'count': 10
                },
            ]
        }
        patchers = u.start_patch()
        self._assert_check_scaling(
            data=data, cluster=cluster,
            expected_message="Composite hostname test-cluster-very-"
                             "very-very-very-very-very-long-ng-name-"
                             "010.novalocal in provisioned cluster exceeds "
                             "maximum limit 64 characters",
            expected_exception=ex.InvalidDataException)
        u.stop_patch(patchers)

    @mock.patch("sahara.utils.api.request_data")
    @mock.patch("sahara.utils.api.bad_request")
    def _assert_cluster_scaling_validation(self,
                                           bad_req=None,
                                           req_data=None,
                                           data=None,
                                           bad_req_i=None):
        m_func = mock.Mock()
        m_func.__name__ = "m_func"
        req_data.return_value = data
        v.validate(c_schema.CLUSTER_SCALING_SCHEMA,
                   self._create_object_fun)(m_func)(data=data,
                                                    cluster_id='42')

        self.assertEqual(1, req_data.call_count)
        self._assert_calls(bad_req, bad_req_i)

    @mock.patch("sahara.utils.api.request_data")
    @mock.patch("sahara.utils.api.bad_request")
    def _assert_cluster_scaling_validation_v2(self,
                                              bad_req=None,
                                              req_data=None,
                                              data=None,
                                              bad_req_i=None):
        m_func = mock.Mock()
        m_func.__name__ = "m_func"
        req_data.return_value = data
        v.validate(c_schema.CLUSTER_SCALING_SCHEMA_V2,
                   self._create_object_fun)(m_func)(data=data,
                                                    cluster_id='42')

        self.assertEqual(1, req_data.call_count)
        self._assert_calls(bad_req, bad_req_i)

    @mock.patch("sahara.service.api.OPS")
    def test_cluster_scaling_scheme_v_resize_ng(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        self._create_object_fun = mock.Mock()
        data = {
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       '{} is not valid under any of the given schemas')
        )
        data = {
            'resize_node_groups': [{}]
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups[0]: 'name' is a required property")
        )
        data = {
            'resize_node_groups': [
                {
                    'name': 'a'
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups[0]: 'count' is a required "
                       "property")
        )
        data = {
            'resize_node_groups': [
                {
                    'name': 'a',
                    'flavor_id': '42',
                    'count': 2
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups[0]: Additional properties are not "
                       "allowed ('flavor_id' was unexpected)")
        )

    @mock.patch("sahara.service.api.OPS")
    def test_cluster_scaling_scheme_v_resize_ng_v2(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        self._create_object_fun = mock.Mock()
        data = {
        }
        self._assert_cluster_scaling_validation_v2(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       '{} is not valid under any of the given schemas')
        )
        data = {
            'resize_node_groups': [{}]
        }
        self._assert_cluster_scaling_validation_v2(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups[0]: 'name' is a required property")
        )
        data = {
            'resize_node_groups': [
                {
                    'name': 'a'
                }
            ]
        }
        self._assert_cluster_scaling_validation_v2(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups[0]: 'count' is a required "
                       "property")
        )
        data = {
            'resize_node_groups': [
                {
                    'name': 'a',
                    'flavor_id': '42',
                    'instances': ['id1'],
                    'count': 2
                }
            ]
        }
        self._assert_cluster_scaling_validation_v2(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups[0]: Additional properties are not "
                       "allowed ('flavor_id' was unexpected)")
        )

    @mock.patch("sahara.service.api.OPS")
    def test_cluster_scaling_validation_add_ng(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        data = {
            'add_node_groups': [
                {
                    "node_group_template_id": "5185a809-6bf7-"
                                              "44ed-9de3-618270550e2c",
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "add_node_groups[0]: {'node_group_template_id': "
                       "'5185a809-6bf7-44ed-9de3-618270550e2c'} "
                       "is not valid under any of the given schemas")
        )
        data = {
            'add_node_groups': [
                {
                    "node_group_template_id": "5185a809-6bf7-"
                                              "44ed-9de3-618270550e2c",
                    'name': 'a'
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "add_node_groups[0]: {'node_group_template_id': "
                       "'5185a809-6bf7-44ed-9de3-618270550e2c', "
                       "'name': 'a'} is not valid under any "
                       "of the given schemas")
        )

    @mock.patch("sahara.service.api.OPS")
    def test_cluster_scaling_validation_right_schema(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        data = {
            'add_node_groups': [
                {
                    "node_group_template_id": "5185a809-6bf7-"
                                              "44ed-9de3-618270550e2c",
                    'name': 'a',
                    'count': 3
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data
        )
        data = {
            'resize_node_groups': [
                {
                    'name': 'a',
                    'count': 3
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data
        )
        data = {
            'resize_node_groups': [
                {
                    'name': 'a',
                    'count': 3
                }
            ],
            'add_node_groups': [
                {
                    "node_group_template_id": "5185a809-6bf7-"
                                              "44ed-9de3-618270550e2c",
                    'name': 'a',
                    'count': 3
                }
            ]
        }
        self._assert_cluster_scaling_validation(
            data=data
        )

    @mock.patch("sahara.service.api.OPS")
    def test_cluster_scaling_scheme_validation_types(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        data = {
            'resize_node_groups': {},
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "resize_node_groups: {} is not of type 'array'")
        )
        data = {
            'add_node_groups': {}
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "add_node_groups: {} is not of type 'array'")
        )
        data = {
            'resize_node_groups': [],
        }
        self._assert_cluster_scaling_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR',
                       'resize_node_groups: [] is too short')
        )

    @mock.patch("sahara.service.api.OPS")
    def test_cluster_scaling_v_right_data(self, ops):
        self.setup_context(tenant_id='t')
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        self._create_object_fun = c_s.check_cluster_scaling

        data = {
            'resize_node_groups': [
                {
                    'name': 'ng',
                    'count': 4
                }
            ],
            'add_node_groups': [
                {
                    'name': 'a',
                    'flavor_id': '42',
                    'node_processes': ['namenode'],
                    'count': 3
                },
            ]
        }
        patchers = u.start_patch()
        self._assert_cluster_scaling_validation(data=data)
        u.stop_patch(patchers)

    @mock.patch("sahara.service.api.OPS")
    def test_check_cluster_scaling_wrong_engine(self, ops):
        ops.get_engine_type_and_version.return_value = "direct.1.1"
        ng1 = tu.make_ng_dict('ng', '42', ['namenode'], 1)
        cluster = tu.create_cluster(
            "cluster1", "tenant1", "fake", "0.1", [ng1],
            status=c_u.CLUSTER_STATUS_ACTIVE, id='12321',
            sahara_info={"infrastructure_engine": "heat.1.1"})

        self._assert_check_scaling(
            data={}, cluster=cluster,
            expected_message="Cluster created with heat.1.1 infrastructure "
                             "engine can't be scaled with direct.1.1 engine")

    @mock.patch("sahara.service.api.OPS")
    def test_check_heat_cluster_scaling_missing_engine(self, ops):
        ops.get_engine_type_and_version.return_value = "heat.1.1"
        ng1 = tu.make_ng_dict('ng', '42', ['namenode'], 1)
        cluster = tu.create_cluster("cluster1", "tenant1", "fake", "0.1",
                                    [ng1], status=c_u.CLUSTER_STATUS_ACTIVE,
                                    id='12321')

        self._assert_check_scaling(
            data={}, cluster=cluster,
            expected_message="Cluster created before Juno release can't be "
                             "scaled with heat.1.1 engine")

    @mock.patch('sahara.utils.openstack.images.image_manager')
    @mock.patch('sahara.utils.openstack.nova.client')
    @mock.patch("sahara.service.api.OPS")
    def test_check_cluster_scaling_missing_resource(self, ops,
                                                    m_nova, m_image):
        ops.get_engine_type_and_version.return_value = "heat.1.1"
        ng1 = tu.make_ng_dict('ng', '42', ['namenode'], 1)

        nova = mock.Mock()
        m_nova.return_value = nova
        nova.keypairs.get.side_effect = u._get_keypair
        cluster = tu.create_cluster(
            "cluster1", "tenant1", "fake", "0.1", [ng1],
            status=c_u.CLUSTER_STATUS_ACTIVE,
            sahara_info={"infrastructure_engine": "heat.1.1"},
            id='12321', user_keypair_id='keypair')
        self._assert_check_scaling(
            data={}, cluster=cluster,
            expected_exception=ex.NotFoundException,
            expected_message="Requested keypair 'keypair' not found")

        image = mock.Mock()
        m_image.return_value = image
        image.list_registered.return_value = [mock.Mock(id='image1'),
                                              mock.Mock(id='image2')]
        cluster = tu.create_cluster(
            "cluster1", "tenant1", "fake", "0.1", [ng1],
            status=c_u.CLUSTER_STATUS_ACTIVE,
            sahara_info={"infrastructure_engine": "heat.1.1"},
            id='12321', default_image_id='image_id',
            user_keypair_id='test_keypair')
        self._assert_check_scaling(
            data={}, cluster=cluster,
            expected_message="Requested image 'image_id' is not registered")



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from sahara.service.api import v10 as api
from sahara.service.validations import node_group_template_schema as ngt_schema
from sahara.service.validations import node_group_templates as nt
from sahara.tests.unit.service.validation import utils as u


class TestNGTemplateCreateValidation(u.ValidationTestCase):
    def setUp(self):
        super(TestNGTemplateCreateValidation, self).setUp()
        self._create_object_fun = nt.check_node_group_template_create
        self.scheme = ngt_schema.NODE_GROUP_TEMPLATE_SCHEMA
        api.plugin_base.setup_plugins()

    def test_node_groups_create_required(self):
        self._assert_create_object_validation(
            data={
            },
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'name' is a required property")
        )
        self._assert_create_object_validation(
            data={
                'name': 'a'
            },
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'flavor_id' is a required property")
        )
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
            },
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'hadoop_version' is a required property")
        )
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1'
            },
            bad_req_i=(1, "VALIDATION_ERROR",
                       u"'node_processes' is a required property")
        )
        self._assert_create_object_validation(
            data={
                'name': "a",
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': []
            },
            bad_req_i=(1, 'VALIDATION_ERROR',
                       'node_processes: \[\] is too short')
        )

    def test_ng_template_create_v_names(self):
        data = {
            'name': 'a',
            'flavor_id': '42',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'node_processes': ['namenode']
        }
        self._assert_valid_name_hostname_validation(data)

    def test_ng_template_create_v_node_processes(self):
        self._assert_create_object_validation(
            data={
                'name': "a",
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ["namenode", "namenode"]
            },
            bad_req_i=(1, 'INVALID_DATA',
                       'Duplicates in node processes have been detected')
        )
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['wrong_process']
            },
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Plugin doesn't support the following node processes: "
                       "['wrong_process']")
        )

    @mock.patch(
        "sahara.service.validations.base.check_volume_availability_zone_exist")
    @mock.patch("sahara.service.validations.base.check_volume_type_exists")
    @mock.patch(
        "sahara.service.validations.base.check_availability_zone_exist")
    @mock.patch("sahara.service.validations.base.check_security_groups_exist")
    def test_ng_template_create_v_right(self,
                                        s_groups, a_zone, v_type, v_a_zone):
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode',
                                   'datanode',
                                   'tasktracker',
                                   'jobtracker'],
                'image_id': '550e8400-e29b-41d4-a716-446655440000',
                'node_configs': {},
                'volumes_per_node': 2,
                'volumes_size': 10,
                'volume_type': 'fish',
                'volumes_availability_zone': 'ocean',
                'volume_mount_prefix': '/tmp',
                'description': "my node group",
                'floating_ip_pool': 'd9a3bebc-f788-4b81-9a93-aa048022c1ca',
                'security_groups': ['cat', 'dog'],
                'auto_security_group': False,
                'availability_zone': 'here',
                'is_proxy_gateway': False,
                'volume_local_to_instance': False,
                'is_public': False,
                'is_protected': False
            }
        )

    def test_ng_template_create_v_nulls(self):
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode',
                                   'datanode',
                                   'tasktracker',
                                   'jobtracker'],

                'image_id': None,
                'node_configs': None,
                'volumes_size': None,
                'volume_type': None,
                'volumes_availability_zone': None,
                'volume_mount_prefix': None,
                'description': None,
                'floating_ip_pool': None,
                'security_groups': None,
                'auto_security_group': None,
                'availability_zone': None,
                'is_proxy_gateway': None,
                'volume_local_to_instance': None,
                'is_public': None,
                'is_protected': None
            }
        )

    def test_ng_template_create_v_minimum_ints(self):
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['wrong_process'],
                'volumes_per_node': -1
            },
            bad_req_i=(1, 'VALIDATION_ERROR',
                       'volumes_per_node: -1(.0)? is less than the minimum '
                       'of 0')
        )
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['wrong_process'],
                'volumes_size': 0
            },
            bad_req_i=(1, 'VALIDATION_ERROR',
                       'volumes_size: 0(.0)? is less than the minimum of 1')
        )

    def test_ng_template_create_v_types(self):
        default_data = {
            'name': 'a', 'flavor_id': '42',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'node_processes': ['namenode']
        }
        self._assert_types(default_data)

    def test_ng_template_create_v_unique_ng(self):
        data = {
            'name': 'test',
            'flavor_id': '42',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'node_processes': ['namenode']}
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, 'NAME_ALREADY_EXISTS',
                       "NodeGroup template with name 'test' already exists")
        )

    def test_ng_template_create_v_flavor_exists(self):
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '1',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode']
            },
            bad_req_i=(1, 'NOT_FOUND',
                       "Requested flavor '1' not found")
        )

    def test_ng_template_create_validate_image(self):
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode'],
                'image_id': '12345'
            },
            bad_req_i=(1, 'VALIDATION_ERROR',
                       "image_id: '12345' is not a 'uuid'")
        )
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode'],
                'image_id': '12345678-1234-1234-1234-123456789000'
            },
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Requested image "
                       "'12345678-1234-1234-1234-123456789000' "
                       "is not registered")
        )
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode'],
                'image_id': '813fe450-40d2-4acc-ade5-ea753a1bd5bc'
            },
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Requested image "
                       "'813fe450-40d2-4acc-ade5-ea753a1bd5bc' "
                       "doesn't contain required tags: "
                       "['0.1']")
        )
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode'],
                'image_id': '550e8400-e29b-41d4-a716-446655440000'
            }
        )

    def test_ng_template_create_v_ng_configs(self):
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode'],
                'node_configs': {
                    'wrong_target': {
                        'mapreduce.task.tmp.dir': '/temp/'
                    }
                }},
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Plugin doesn't contain applicable "
                       "target 'wrong_target'")
        )
        self._assert_create_object_validation(
            data={
                'name': 'test-ng',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['namenode'],
                'node_configs': {
                    'general': {
                        's': 'a'
                    }
                }
            },
            bad_req_i=(1, 'INVALID_REFERENCE',
                       "Plugin's applicable target 'general' doesn't "
                       "contain config with name 's'")
        )

    def test_ng_template_cinder(self):
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['wrong_process'],
                'volumes_per_node': -1
            },
            bad_req_i=(1, 'VALIDATION_ERROR',
                       'volumes_per_node: -1(.0)? is less than the minimum '
                       'of 0')
        )
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['wrong_process'],
                'volumes_size': 0
            },
            bad_req_i=(1, 'VALIDATION_ERROR',
                       'volumes_size: 0(.0)? is less than the minimum of 1')
        )
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['datanode', 'tasktracker'],
                'volumes_per_node': 1,
                'volumes_size': 1,
                'volume_mount_prefix': '/mnt/volume'
            }
        )
        data = {
            'name': 'a',
            'flavor_id': '42',
            'plugin_name': 'fake',
            'hadoop_version': '0.1',
            'node_processes': ['datanode', 'tasktracker'],
            'volumes_per_node': 1,
            'volumes_size': 1,
            'volume_mount_prefix': 'qwerty'
        }
        self._assert_create_object_validation(
            data=data,
            bad_req_i=(1, 'VALIDATION_ERROR', "volume_mount_prefix: 'qwerty' "
                                              "is not a 'posix_path'")
        )

    def test_wrong_floating_ip_pool(self):
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['datanode', 'tasktracker'],
                'floating_ip_pool': 'network_bad'
            },
            bad_req_i=(1, 'NOT_FOUND', "Floating IP pool network_bad "
                                       "not found")
        )

    def test_volumes_per_node_without_volumes_size(self):
        self._assert_create_object_validation(
            data={
                'name': 'a',
                'flavor_id': '42',
                'plugin_name': 'fake',
                'hadoop_version': '0.1',
                'node_processes': ['datanode', 'tasktracker'],
                'volumes_per_node': 1
            },
            bad_req_i=(1, 'INVALID_REFERENCE', "You must specify a "
                                               "volumes_size parameter")
        )
